//
// Gertbot GUI
//
// Basic GUI 
// (constructor,  de-structor)  
//

#include <stdio.h>

#include "gertbot.h"
#include "gertbot_defines.h"

extern bool uart_opened;

gertbot::gertbot(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags)
{ int b,m;
#ifndef _WIN32
  int status;
#endif
  QString str;
  ui.setupUi(this);
  this->setStatusBar(0); // no status bar
  suppress_updates = false; 
  force_command = false;
  polling = false;
  poll_time = 1000; // 1 second 
  log_window = NULL;
  open_log_on_first_connect = true;

#ifdef _WIN32
  // Running on windows 
  available_commports = available_com_ports();
  if (available_commports==0)
    ui.pd_comport->insertItem(0,"<no comm ports>");
  else
  {
    for (b=0; b<32; b++)
    { if (available_commports & (1<<b))
      { str.sprintf("Com %d",b+1); // bits 0-31 are com 1-32
        ui.pd_comport->insertItem(b,str);
      }  
    }
  }
#else
  ui.pd_comport->hide();
/*
  // On Pi check availbility of uart 
  status = system("./enable_uart cf");
  if (status==1)
  { fprintf(stderr,"The uart does not seem to be available\n");
    fprintf(stderr,"This program can not run without access to the uart\n");
    fprintf(stderr,"use 'sudo enable_uart -cfr' to enable the uart and reboot'\n");
    fprintf(stderr,"use 'enable_uart' to get more information about the program\n");
    exit(1);
  }
  */
#endif

  // Set all 'group' pointers to NULL
  // This indicates GUI not present
  for (b=0; b<4; b++)
  { gui_disp_motor[b] = true;
    board_gui[b].pb_board_to_motor = NULL;
    board_gui[b].pb_motor_to_board = NULL;
    for (m=0; m<4; m++)
      motor_gui[b][m].uipnt = NULL;

/*
    board[b].present=false;
    board[b].adc_enable = 0x0F;
    board[b].dac_enable = 0x00;
    board[b].sync_mode = false;
    board[b].input_ports = 0x000063FF; // DAC, !ADC, spares, EXT
    board[b].output_ports = 0x0;
    board[b].outputs_status = 0x0;
*/
    text_display[b]=0;
  }
  // the four board tabs 
  tab[0] = new QWidget();
  tab[1] = new QWidget();
  tab[2] = new QWidget();
  tab[3] = new QWidget();

  frm_board[0] = ui.frm_board0;
  frm_board[1] = ui.frm_board1;
  frm_board[2] = ui.frm_board2;
  frm_board[3] = ui.frm_board3;

  // Get Icons from resources in executable
  // (generated using rcc _)
  icon_left.addFile(QString::fromUtf8(":/gertbot/Resources/left.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_right.addFile(QString::fromUtf8(":/gertbot/Resources/right.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_pause.addFile(QString::fromUtf8(":/gertbot/Resources/pause.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_stop.addFile(QString::fromUtf8(":/gertbot/Resources/stop.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_left_grn.addFile(QString::fromUtf8(":/gertbot/Resources/left_grn.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_right_grn.addFile(QString::fromUtf8(":/gertbot/Resources/right_grn.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_pause_grn.addFile(QString::fromUtf8(":/gertbot/Resources/pause_grn.ico"), QSize(), QIcon::Normal, QIcon::Off);
  icon_stop_grn.addFile(QString::fromUtf8(":/gertbot/Resources/stop_grn.ico"), QSize(), QIcon::Normal, QIcon::Off);

  dcc_timer.setSingleShot(true);

  ui.pb_halt->setStyleSheet("QPushButton { color : red}");

  connect(ui.pb_connect,SIGNAL(clicked(bool)), this, SLOT(connect_slot()));
  connect(ui.pb_stop_all,SIGNAL(clicked(bool)), this, SLOT(stop_all_slot()));
  connect(ui.pb_halt,SIGNAL(clicked(bool)), this, SLOT(halt_slot()));
  connect(ui.pb_read_status ,SIGNAL(clicked(bool)), this, SLOT(read_status_slot()));
  connect(ui.pb_read_errors ,SIGNAL(clicked(bool)), this, SLOT(read_errors_slot()));
  connect(ui.pb_sync ,SIGNAL(clicked(bool)), this, SLOT(do_sync_slot()));
  connect(ui.cb_log  ,SIGNAL(toggled(bool)), this, SLOT(log_slot(bool)));
//  connect(ui.pb_resend  ,SIGNAL(clicked(bool)), this, SLOT(resend_slot()));
  connect(ui.cb_poll  ,SIGNAL(toggled(bool)), this, SLOT(poll_slot(bool)));
  connect(ui.sb_polltime  ,SIGNAL(valueChanged(double)), this, SLOT(poll_time_slot(double)));
  connect(&timer,     SIGNAL(timeout()), this, SLOT(timer_slot()    )  );
  connect(&dcc_timer, SIGNAL(timeout()), this, SLOT(dcc_timer_slot()  ) );
  connect(ui.pb_send  ,SIGNAL(clicked(bool)), this, SLOT(user_send_slot()));

  dcc_init();

} // constructor

gertbot::~gertbot()
{
  if (uart_opened)
  {
    close_uart();
    uart_opened = false;
  }
  if (polling)
    timer.stop();
} // ~gertbot



//
// Switch log window on/off
// 
void gertbot::log_slot(bool on)
{ if (suppress_updates)
    return;
  open_log_on_first_connect = false; 
  if (on)
  { if (!log_window)
    { log_window = new gertbot_log(this,0);
      if (!log_window)
        return;
    }
    // start log window somewhere to right of this one 
    log_window->setGeometry(this->x()+this->width()+20,this->y()+25,
                            log_window->width(),log_window->height());
    log_window->show();
  }
  else
  { if (!log_window)
      return; 
    delete(log_window);
    log_window = NULL;
  }
} // log on/off 

// User closed log window
void gertbot::log_close_slot()
{ suppress_updates = true;
  ui.cb_log->setChecked(false);
  suppress_updates = false;
}


//
// Mode changed
// Update the gui 
//
void gertbot::pd_mode_slot()
{ QObject *qto = (QObject *)sender();
  int b,m,index;
  byte id,mode;

  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  index = motor_gui[b][m].uipnt->pd_mode->currentIndex();
  mode = index_to_mode[index];
  // Start with basic display mode
  motor_gui[b][m].more_index = 0;
  set_motormode_gui(b,m); 
  id = (b<<2) | m;

  gb_motor[b][m].mode = mode;
  switch (mode)
  {
  case MOT_MODE_OFF:
    // Stop 
    uart_tx(CMD_OPMODE,id,mode);
    break;

  case MOT_MODE_BRUSH:
    // brushed mode 
    uart_tx(CMD_OPMODE,id,mode);
    send_freq(b,m);
    break;


  case MOT_MODE_STEP_GRAY:
  case MOT_MODE_STEP_PULSE:
  case (MOT_MODE_STEP_GRAY|MOT_MODE_STEP_HOLD):
  case (MOT_MODE_STEP_PULSE|MOT_MODE_STEP_HOLD):
    // set mode 
    uart_tx(CMD_OPMODE,id,mode);
    send_freq(b,m);
    break;
  case MOT_MODE_DCC:
    // First switch channel short circuit to off
    uart_tx(CMD_STOPSHORT,id,ENDSTOP_OFF|ENB_ERR_IGNORE); // No short no endstops 
    // DCC mode  
    uart_tx(CMD_OPMODE,id,mode);
    break;
  default:
    fprintf(stderr,"pd_mode_slot illegal mode");
  } // switch

} // pd_mode_slot

//
// very primitive log window
//
void gertbot::add_log(QString str)
{ 
  if (!log_window)
    return;
  log_window->add_log(str);
} // add_log


void gertbot::update_slot()
{ int  b,m,i;
  byte id;
  for (b=0; b<4; b++)
  { if (board[b].present)
    {
      for (m=0; m<4; m++)
      { i = motor_gui[b][m].uipnt->pd_mode->currentIndex();
        switch (i)
        {
        case 0  : break;
        default :
          id = (b<<2) | m;
          uart_tx(CMD_OPMODE,id,index_to_mode[i]);
          send_freq(b,m);
          if (i==1)
          { send_dutycycle(b,m);
          }
          else
          { // send_steps(b,m);
          }
          break;
        }
      } // for each motor 
    } // if boad preent
  } // for each board
} // update_slot

void gertbot::poll_slot(bool on)
{ int b,b_count;
  if (suppress_updates)
    return; 
  if (on)
  { 
    // check if there ARE any boards 
    b_count = 0;
    for (b=0; b<4; b++)
      if (board[b].present)
        b_count++;
    if (b_count==0)
    { // No there are not.
      suppress_updates = true;
      ui.cb_poll->setChecked(false);
      suppress_updates = false;
      return;
    }
    timer.start(poll_time);
    polling = true;
  }
  else
  {
    timer.stop();
    polling = false;
  }
}

void gertbot::poll_time_slot(double pt)
{
  poll_time = (int)(pt * 1000.0);
  poll_slot(ui.cb_poll->isChecked());
}

void gertbot::timer_slot()
{ byte mess[5],buffer[64];
  static byte brd=-1;
  QString str;
  mess[0]=CMD_POLL;
  mess[2]=0x50;
  mess[3]=0x50;
  mess[4]=0x50;
  // next board
  do {
    brd++;
    if (brd>=4)
      brd = 0;
  } while (!board[brd].present);

  if (board[brd].present)
  { mess[1] = brd<<2;
    uart_tx(mess[0],mess[1],mess[2],mess[3],mess[4]);
    if (!get_reply(buffer,4,brd<<2,(byte)CMD_POLL))
    { 
      str.sprintf("Board %d: error polling",brd);
      add_log(str);
      return;
    }
    // printf("0x%02X ",buffer[2]); fflush(stdout);
    // process poll result 
    if (buffer[2] & 0x01)
// HALT bit ? 
      frm_board[brd]->setStyleSheet("QFrame { background-color : red }");
    else
      if (buffer[2] & 0xF0)
 // error bits ? 
        frm_board[brd]->setStyleSheet("QFrame { background-color : orange }");
      else
        if (buffer[2] & 0x02)
 // error queue? 
          frm_board[brd]->setStyleSheet("QFrame { background-color : yellow }");
        else
          frm_board[brd]->setStyleSheet("QFrame { background-color : green }");
  } // if board present

} // timer slot

// 
// Low level send arbitrary user data 
//
void gertbot::user_send_slot()
{ unsigned char buffer[128];
  char text[128];
  int n,i,hex,l;
  l = ui.pd_user_command->currentText().length();

  for (i=0; i<l && i<128; i++)
    text[i] = ui.pd_user_command->currentText().at(i).cell();
  text[i]=0;
  n=0;
  for (i=0; i<l && i<128; i++)
  {
    if (isxdigit(text[i]))
    { if (text[i]<='9')
        hex = text[i]-'0';
      else
      { if (text[i]<='F')
          hex = text[i]-'A'+10;
       else
          hex = text[i]-'a'+10;
      }
      if (n & 1)
        buffer[n>>1] |= hex;
      else
        buffer[n>>1] = hex<<4;
      n++;
    } // is xdigit
  } // for each char
  write_uart(buffer,n>>1);
} // user_send_slot

