//
// Gertbot GUI
//
// Boards control 
//


#include <stdio.h>

#include "gertbot.h"

extern bool uart_opened;

//
// Get status of everything
// Update gui accordingly
//
void gertbot::read_status_slot()
{ int b,m;
  QString str;
  for (b=0; b<4; b++)
  {
    if (board[b].present)
    {
      if (!get_board_status(b,&(board[b])))
      {
        str.sprintf("Board %d: error getting status",b);
        add_log(str);
      }
      else
      {
        if (!get_io_status(b,&(board[b])))
        {
          str.sprintf("Board %d: error getting I/O",b);
          add_log(str);
        }
        else
        {
          set_gui_from_board(b);
        }
      }

      for (m=0; m<4; m++)
      {
        if (!get_motor_config(b,m,&(gb_motor[b][m])))
        { 
          str.sprintf("Board %d motor %d: error getting config",b,m);
          add_log(str);
          continue;
        }       
        // use the newly quired data to update the gui
        set_gui_from_motor(b,m);

      } // for each motor 
    } // if board operational
  } // for each board
} // get gertbot status

// System error values 
#define MAX_ERROR 33
const char *error_mess[MAX_ERROR+1] = {
"No error"                                       , // 0x0000
"Command input buffer overflow "                 , // ERROR_INP_OVERFLOW 0x0001 // 
"Program internal error "                        , // ERROR_FATALSYSTEM  0x0002 // 
"Mode command value error "                      , // ERROR_MODEERR      0x0003 // 
"End stop parameter wrong "                      , // ERROR_ENDSTOP      0x0004 // 
"Lin. freq error: Motor was not in linear mode " , // ERROR_LINFREQ      0x0005 // 
"Duty cycle error: Motor was not in linear mode" , // ERROR_LINDC        0x0006 // 
"Illegal linear freq given "                     , // ERROR_ILLPWMFREQ   0x0007 // 
"Illegal duty cycle given "                      , // ERROR_ILLPWMDC     0x0008 // 
"Brushed motor start but not given freq."        , // ERROR_NOLINFREQ    0x0009 // 
"Stepper motor start but not given freq."        , // ERROR_NOSTEPFREQ   0x000A // 
"Step command: Motor was not in step mode "      , // ERROR_STEP         0x000B // 
"Step freq. command: Motor was not in step mode" , // ERROR_STEPFREQ     0x000C // 
"Illegal end stop command "                      , // ERROR_ENDSTOPERR   0x000D // 
"Command conflicts with mode "                   , // ERROR_MODECONFL    0x000E // 
"End stop polarity command error "               , // ERROR_ENDSPOL      0x000F // 
"Start command given with halt active "          , // ERROR_HALTACTIVE   0x0010 // 
"Timer queue pool empty "                        , // ERROR_TQ_EMPTY     0x0011 // 
"Serial output queue overflow "                  , // ERROR_SEROUTOVFL   0x0012 // 
"Illegal stepper frequency "                     , // ERROR_ILLSTEPFREQ  0x0013 // 
"Write to DAC which is disabled "                , // ERROR_DACDIS       0x0014 // 
"Read from ADC which is disabled "               , // ERROR_ADCDIS       0x0015 // 
"Enable ADC illegal mask bits "                  , // ERROR_ADCMASK      0x0016 // 
"Enable DAC illegal mask bits "                  , // ERROR_DACMASK      0x0017 // 
"Linear on/off direction but not a lin. motor "  , // ERROR_LINON        0x0018 // 
"Illegal command "                               , // ERROR_ILLCOMMAND   0x0019 // 
"HALT was active "                               , // ERROR_HALTSEEN     0x001A // 
"Enable 0 was negated "                          , // ERROR_A_SEEN       0x001B // 
"Enable 1 was negated "                          , // ERROR_B_SEEN       0x001C // 
"Enable 2 was negated "                          , // ERROR_C_SEEN       0x001D // 
"Enable 3 was negated "                          , // ERROR_D_SEEN       0x001E // 
"DCC queue overflow "                            , // ERROR_DCC_OVFL     0x001F // 
"DCC command length wrong "                      , // ERROR_DCC_MESS     0x0020 // 
"Illegal error status code"
};

//
// Read error status and add to log 
//
void gertbot::read_errors_slot()
{ int b;
  uint32_t value,repeat;
  byte buffer[64];
  byte id;
  QString str;
  for (b=0; b<4; b++)
  {
    if (board[b].present)
    { repeat = 0;
      id = (b<<2);
      do {
        uart_tx((byte)CMD_GET_ERROR,id,0x50,0x50);
        if (!get_reply(buffer,4,id,(byte)CMD_GET_ERROR))
        { str.sprintf("%d: No reply on get error",b);
          add_log(str);
          continue;
        }
        // get 2 byte error code 
        value  = buffer[2];
        value |= (value<<8) | buffer[3];
        if (value>=MAX_ERROR)
           str.sprintf("Board %d: Illegal error code 0x%04X",b,value);
        else
           str.sprintf("Board %d: %s",b,error_mess[value]);
        add_log(str);
        repeat++;// count answers 
        // Stop if the error code=0
        // or after 16 which is the maximum possible 
      } while (value!=0 && repeat<16);
    } // if board is present 
  } // for each board 
} // get error status 


//
// Check which boards are present
// Make controls for boards which are present
//
// Not yet done: 
// what happens if board(s) appear/disappear when pressing
// the connect button again...
// 
//
void gertbot::connect_slot()
{ int b,m,num_boards;
  int version_ms,version_ls;
  int local_board_present[4];
  QString str;

  // send group is disabled at start to prevent distraction
  // This way users are more likely to use the 'connect' button
  ui.grp_send->setEnabled(true);
  if (open_log_on_first_connect)
  { open_log_on_first_connect=false;
    if (!ui.cb_log->isChecked())
      ui.cb_log->setChecked(true);
  }

#ifdef _WIN32
  // Need major work for closing/reopen!!
  int p;
  if (uart_opened)
  { close_uart();
    uart_opened=false;
  }
  p = ui.pd_comport->currentIndex()+1;
  b = index_comport(available_commports,p);

  if (open_uart(b))
  { 
    add_log("Fatal! Could not open com port");
    return;
  }

  if (setup_comm_port(L"57600,n,8,1"))
  {
    add_log("Fatal! Could not config com port");
    return;
  }

#else

  if (open_uart(1))
  { 
    add_log("Fatal! Could not open uart");
    add_log("Check uart availability (use enable_uart)");
    return;
  }
#endif

  uart_opened = true;
  add_log("Searching for boards");
  num_boards = 0;
  for (b=0; b<4; b++)
  { str.sprintf("Board %d...",b);
    add_log(str);
//    ui.te_log->update();
    QApplication::processEvents();
//    ui.te_log->moveCursor(QTextCursor::End);
    if (!find_board(b,&version_ms,&version_ls)) 
    { local_board_present[b]=false;
      add_log(" Not found");
    }
    else
    { local_board_present[b]=true;
      num_boards++;
      str.sprintf(" Found version %d.%d",version_ms,version_ls);
      add_log(str);
      // Get build data & time 
      // str=get_build_date_time(b);
      // add_log(str);
    }
    QApplication::processEvents();
  }
//  ui.te_log->moveCursor(QTextCursor::End);
      
  if (num_boards==0)
  { str.sprintf("No boards found!!\n");
    add_log(str);
    for (b=0; b<4; b++)
      board[b].present = local_board_present[b];
    return;
  }
  ui.pb_read_status->setEnabled(true);
  ui.pb_read_errors->setEnabled(true);
//  ui.pb_update     ->setEnabled(true);
  ui.pb_sync       ->setEnabled(true);

  num_boards = 0;
  for (b=0; b<4; b++)
  { if (local_board_present[b])
    { str.sprintf("Board %d",b);
      // insert TAB in the right place 
      ui.tabWidget->insertTab(num_boards++,tab[b],str);
      // Make the four motor controls
      for (m=0; m<4; m++)
        make_motor_group(b,m);
      make_motor_switch(b);

      // and the board controls 
      make_board_gui(b);
    }
    else
      tab[b]->hide(); // setStyleSheet("QTabBar { color : red; }");
  }
  ui.tabWidget->setCurrentIndex(0);
  // NOW tell users which boards are present
  // if not they can press update/error button and causes system crash
  for (b=0; b<4; b++)
    board[b].present = local_board_present[b];

} // connect

//
// This is not suported as the GCC compiler for Atmel does
// not recognise the __DATE__ as a constant
// 
QString gertbot::get_build_date_time(int brd)
{ QString str;
  int timestamp,year,month,day,hour,min;
  byte id,rec[5],tx_buf[2]; 
  id = (brd<<2);
  tx_buf[0] = CMD_BUILDDT;
  tx_buf[1] = id;
  uart_tx(tx_buf,2,5);
  // uart Get data back.... 
  if (!get_reply(rec,5,id,(byte)CMD_BUILDDT))
  { str.sprintf("%d:No reply on read timestamp %d",brd);
    return str;
  }
  timestamp = (rec[2]<<16) + (rec[3]<<8) + (rec[4]);
  year       = timestamp / (12*31*24*60);
  timestamp -= year      * (12*31*24*60);
  month      = timestamp / (31*24*60);
  timestamp -= month     * (31*24*60);
  day        = timestamp / (24*60);
  timestamp -= day       * (24*60);
  hour       = timestamp / (60);
  timestamp -= hour      * (60);
  min = timestamp;

  str.sprintf("%2d %2d %4d, %2d:%02d",day,month,year+2014,hour,min);

  return str;
} // get_build_date_time


//
// Switch open drains on/off
//
void gertbot::opendrain_slot()
{ QObject *qto = (QObject *)sender();
  int b;
  byte id,mode;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  id = (b<<2);
  mode  = boardui[b].cb_opendrain0->isChecked() ? 0x01 : 0x00;
  mode |= boardui[b].cb_opendrain1->isChecked() ? 0x02 : 0x00;
  uart_tx((byte)CMD_OD,id,mode);
} // opendrain_slot

//
// Send ADC and DAC enable/disable command
//
void gertbot::adc_dac_enable(int b)
{ byte id,adc_enb,dac_enb;
  bool adc_dac_change;
  id = b<<2;
  dac_enb  = j3_gui[b].pd[15]->currentIndex()==GUI_PD_FUNCT ? 0x01 : 0x00; // DAC0
  dac_enb |= j3_gui[b].pd[14]->currentIndex()==GUI_PD_FUNCT ? 0x02 : 0x00; // DAC1
  adc_enb  = j3_gui[b].pd[11]->currentIndex()==GUI_PD_FUNCT ? 0x02 : 0x00; // ADC1
  adc_enb |= j3_gui[b].pd[10]->currentIndex()==GUI_PD_FUNCT ? 0x01 : 0x00; // ADC0
  adc_enb |= j3_gui[b].pd[13]->currentIndex()==GUI_PD_FUNCT ? 0x08 : 0x00; // ADC3
  adc_enb |= j3_gui[b].pd[12]->currentIndex()==GUI_PD_FUNCT ? 0x04 : 0x00; // ADC2
  adc_dac_change = false;
  if (board[b].adc_enable != adc_enb)
    adc_dac_change = true;
  if (board[b].dac_enable != dac_enb)
    adc_dac_change = true;

  if (adc_dac_change || force_command)
  { board[b].adc_enable = adc_enb;
    board[b].dac_enable = dac_enb;
    if (suppress_updates)
      return;
    uart_tx((byte)CMD_SETADCDAC,id,adc_enb,dac_enb);
  }
} // adc_dac_enable
 
//
// Enable/disable a DAC
//
void gertbot::dac_enable_slot()
{ QObject *qto = (QObject *)sender();
  int b;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  adc_dac_enable(b);
} //dac_enable_slot

//
// Set (send) DAC value
// Beware that DAC has limited range: 1/6 Vref..5/6 Vref
// I assume this means iof the outpuit is set to 0x0000
// the Vout = 1/6 Vref
//
void gertbot::dac_value_slot() 
{ QObject *qto = (QObject *)sender();
  byte id,st[3];
  int b,d,i_value;
  double d_value;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  d = qto->objectName()[1].digitValue();
  
  d_value = j3_gui[b].sb_dac[d]->value();
  board[b].dac[d]=d_value;
  // Map value from control onto 12 bit range
  d_value -= DAC_MIN;
  i_value = (int)(d_value * 4095.0/(DAC_MAX-DAC_MIN)+0.5);
 
  id = (b <<2) | d;
  st[0] = (byte)((i_value>> 8) & 0xFF);
  st[1] = (byte)( i_value      & 0xFF);
  uart_tx((byte)CMD_DAC,id,st[0],st[1]);
} // dac_value_slot


//
// Enable/disable a ADC
//
void gertbot::adc_enable_slot()
{ QObject *qto = (QObject *)sender();
  int b;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  adc_dac_enable(b);
} // adc_enable_slot


//
// Get ADC value
// 
float gertbot::adc_read(int brd,int adc)
{ byte id,rec[64];
  int  i_value;
  float f_value;
  QString str;
 
  id = (brd<<2) | adc;
  uart_tx((byte)CMD_GET_ADC,id,0x50,0x50,0x50);
  // uart Get data back.... 
  if (!get_reply(rec,4,id,(byte)CMD_GET_ADC))
  { str.sprintf("%d:No reply on read ADC %d",brd,adc);
    add_log(str);
    return -1.0;
  }
  i_value = rec[2];
  i_value = (i_value<<8) | rec[3];
  if (i_value==0xFFFF)
  { board[brd].adc[adc]=-1;
    return -1;
  }
  f_value = i_value * 3.3/4095;
  board[brd].adc[adc] = f_value;
  return f_value;
} // adc_read

//
// read input pins 
//
int gertbot::input_read(int brd)
{ byte id,rec[64];
  int  i_value;
  QString str;
 
  id = (brd<<2);
  uart_tx((byte)CMD_READIO,id,0x50,0x50,0x50);
  // uart Get data back.... 
  if (!get_reply(rec,5,id,(byte)CMD_READIO))
  { str.sprintf("%d:No reply on read Input",brd);
    add_log(str);
    return 0;
  }
  i_value  = rec[4];
  i_value |= rec[3]<<8;
  i_value |= rec[2]<<16;
  board[brd].inputs_status = i_value;
  return i_value;
} // input_read


void gertbot::short_slot(int mode)
{ QObject *qto = (QObject *)sender();
  int b,m;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  gb_motor[b][m].short_hot_stop = mode;
  send_endmode_short(b,m);
} // short_slot

void gertbot::sync_mode_slot(int on)
{ QObject *qto = (QObject *)sender();
  int b;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  board[b].sync_mode = on ? true : false;
  board_set_mode(b);
} // sync_mode_slot

void gertbot::attention_slot(int v)
{ QObject *qto = (QObject *)sender();
  int b;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  board[b].attentionmode = v;
  board_set_mode(b);
} // attention_slot

//
// Send board configure command
// Three fields
// Todo: move short/hort mode to separate motor setup command....
//
void gertbot::board_set_mode(int b)
{ byte id,mode[3];
  id = (b<<2);
  mode[0]  = board[b].sync_mode ? 0x01 : 0x00;
  mode[0] |= board[b].attentionmode << 1;
  mode[0] |= gb_motor[b][0].short_hot_stop << 4;
  mode[0] |= (gb_motor[b][1].short_hot_stop << 7) & 0x80;
  mode[1]  =  gb_motor[b][1].short_hot_stop >> 1;
  mode[1] |=  gb_motor[b][2].short_hot_stop << 2;
  mode[1] |=  gb_motor[b][3].short_hot_stop << 5;
  mode[2]  = 0;
  uart_tx((byte)CMD_CONFIGURE,id,mode[0],mode[1],mode[2]);
} // board_set_mode

//
// J3 check box changed
//
void gertbot::j3cb_slot(int checked)
{ QObject *qto = (QObject *)sender();
  int b,p,m;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  p = qto->objectName()[1].toAscii()-'0';

  if (p<=7)
  { // end point: can have two tick box meanings 
    switch (j3_gui[b].pd[p]->currentIndex())
    {
    case GUI_PD_FUNCT :  j3_gui[b].cb[p]->setText(checked ? "High":"Low");
             // motor control tick box
             // That also triggers the data transfer
             m = p/2;
             if (p & 1)
               motor_gui[b][m].uipnt->cb_high_B->setChecked(checked);
             else
               motor_gui[b][m].uipnt->cb_high_A->setChecked(checked);
             break;
    case GUI_PD_INPUT : /* Input: not possible to */ break;
    case GUI_PD_OUTPUT : set_port_out_high(b); 
             j3_gui[b].cb[p]->setText(checked ? "1":"0");
             break;
    }
  }
  else
  { // All the others must be output
    set_port_out_high(b); 
    j3_gui[b].cb[p]->setText(checked ? "1":"0");
  }
  
} // j3cb_slot

//
// Set an output port high or low 
//
// The 'pin swap' is achieved by the control name:
// Pin[0] has j3_gui[b].control[1]
// Pin[1] has j3_gui[b].control[0]
//
void gertbot::set_port_out_high(int b)
{ uint32_t high_bits;
  byte bit,io[3];

  // using J3 mode 
  high_bits = 0;
  for (bit=0; bit<14; bit++)
  { 
    if (j3_gui[b].pd[bit]->currentIndex()==GUI_PD_OUTPUT &&
        j3_gui[b].cb[bit]->isChecked())
      high_bits |= (1<<bit);
  }
  // Swap DAC bits 
  if (j3_gui[b].pd[14]->currentIndex()==GUI_PD_OUTPUT &&
      j3_gui[b].cb[14]->isChecked())
    high_bits |= (1<<15);
  if (j3_gui[b].pd[15]->currentIndex()==GUI_PD_OUTPUT &&
      j3_gui[b].cb[15]->isChecked())
    high_bits |= (1<<14);
   
  board[b].outputs_status = high_bits;

  io[2] = 0xFF;
  io[1] = (high_bits & 0x00FF00) >> 8;
  io[0] = high_bits & 0x0000FF;

  uart_tx(CMD_WRITEIO,b<<2,io[2],io[1],io[0]);

} // set_port_out_high

//
// Set input/output pin mode
// 0 is output
// 1 is input
// Take status from J3 controls 
//
void gertbot::set_io(int b)
{ byte io[3];
  int  bit;
  uint32_t gui_in,gui_out;
  // using J3 mode 

  gui_in = 0;
  for (bit=0; bit<14; bit++)
  { 
    if (j3_gui[b].pd[bit]->currentIndex()==GUI_PD_INPUT)
      gui_in |= (1<<bit);
  }
  // swap DAC bits 
  if (j3_gui[b].pd[14]->currentIndex()==GUI_PD_INPUT)
    gui_in |= 0x8000;
  if (j3_gui[b].pd[15]->currentIndex()==GUI_PD_INPUT)
    gui_in |= 0x4000;

  gui_out = 0;
  for (bit=0; bit<14; bit++)
  { 
    if (j3_gui[b].pd[bit]->currentIndex()==GUI_PD_OUTPUT)
      gui_out |= (1<<bit);
  }
  // swap DAC bits 
  if (j3_gui[b].pd[14]->currentIndex()==GUI_PD_OUTPUT)
    gui_out |= 0x8000;
  if (j3_gui[b].pd[15]->currentIndex()==GUI_PD_OUTPUT)
    gui_out |= 0x4000;


  if (suppress_updates)
    return;

  if (board[b].input_ports !=gui_in  || 
      board[b].output_ports!=gui_out ||
      force_command)
  { board[b].input_ports  =gui_in;
    board[b].output_ports =gui_out;
    io[2] = 0xFF; 
    io[1] = (gui_in & 0x00FF00) >> 8;
    io[0] = (gui_in & 0x0000FF);
    uart_tx(CMD_SETIO,b<<2,io[2],io[1],io[0]);
  }

} // set_io

//
// Read button pressed on board
// Get input status, and ADC values 
//
void gertbot::board_read_slot()
{ QObject *qto = (QObject *)sender();
  byte b,u;
  int i_val,i,bit;
  QString str;
  float  f_val;
  b = qto->objectName()[0].digitValue();
  // split into individual reads

  // read four ADC's
  for (u=0; u<4; u++)
  { 
    if (board[b].adc_enable & (1<<u)) 
    { f_val = adc_read(b,u);
      if (f_val>=0)
      { str.sprintf("%1.2fV",f_val);
        j3_gui[b].lbl_adc[u]->setText(str);
      }
      else
        j3_gui[b].lbl_adc[u]->setText("???");
    }
  }

  // Read I/O's
  i_val = input_read(b);
  // Endstops
  for (i=0; i<8; i++)
  { 
    if (j3_gui[b].pd[i]->currentIndex()==GUI_PD_INPUT) 
    { // only change if in input mode 
      j3_set_input_cb(b,i,i_val & (1<<i));
    }
  }

  // Spares
  for ( ; i<10; i++)
  { 
    if (j3_gui[b].pd[i]->currentIndex()==GUI_PD_INPUT) 
    { // only change if in input mode 
      j3_set_input_cb(b,i,i_val & (1<<i));
    }
  }

  // ADCs & DACs
  bit = 10;
  for (; i<16; i++)
  { 
    if (j3_gui[b].pd[i]->currentIndex()==GUI_PD_INPUT) 
    { // only change if in input mode 
      j3_set_input_cb(b,i,i_val & (1<<i));
    }
    bit++;
  }

} // board_read_slot

//
// Get board status (configuration)
// returns 6 bytes 
//
bool gertbot::get_board_status(int brd,gb_board_type *status)
{ byte id,rec[8],cmd[2];
  QString str;

  if (!status->present)
    return false; // should not happen...

  id = (brd<<2);
  cmd[0] = CMD_BRD_STATUS;
  cmd[1] = id;
  uart_tx(cmd,2,8); 
  // Get reply...
  if (!get_reply(rec,8,id,(byte)CMD_BRD_STATUS))
  { str.sprintf("%d:No reply on read Board",brd);
    add_log(str);
    return false;
  }
  // 2 unused bits 
  status->sync_mode         =  rec[2] & 0x01;
  status->attentionmode     = (rec[2]>>1) & 0x07;
  // rec[3..5] are spare
  return true;
} // get_board_status


//
// Get board I/O status (configuration)
// returns 16 bytes 
//
bool gertbot::get_io_status(int brd,gb_board_type *status)
{ byte id,rec[16],cmd[2];
  QString str;
  if (!status->present)
    return false; // should not happen...
 
  id = (brd<<2);
  cmd[0] = CMD_IO_STATUS;
  cmd[1] = id;
  uart_tx(cmd,2,16);
  // Get reply...
  if (!get_reply(rec,16,id,(byte)CMD_IO_STATUS))
  { str.sprintf("%d:No reply on read I/O",brd);
    add_log(str);
    return false;
  }

  status->adc_enable  = rec[2] & 0x0F;
  status->dac_enable   = rec[3] & 0x03;

  // rec[4] unused
  status->input_ports = rec[6] | (rec[5]<<8);

  // rec[7] unused
  status->output_ports = rec[9] | (rec[8]<<8);
 
  // rec[10] unused
  status->outputs_status = rec[12] | (rec[11]<<8);

  status->od  = rec[13] & 0x03; 

  // rec[14,15] unused

  return true;
} // get_io_status

