//
// DCC related routines
//

/*
 DCC spec says:
   a ONE  takes 2x58  = 116 micro seconds (116e-6)
   a ZERO takes 2x100 = 220 micro seconds (200e-6) nominally!
   The pre-amble is at least 14 'ones'
   Each byte is precede by a zero.
   Packets should be send repeatedly.
   Packets are between 3 and 6 bytes
   Packet for the same destination should be 5ms apart
   (Measured end of first to start of next)
   An idle packet with pre-amble is (14+16)*116 + (8+3)*200 = 5.6mill sec.

   The following is a packet lenght/repeat time table
   (Assume average same number of zeros and ones in the data section)
   Packet                                 Repeat
   length   Ones  Zeros  Time (ms)   4      7      10
     3       26     15     6.01     24.1   42.1   60.2
     4       30     20     7.48     29.9   52.4   74.8
     5       34     25     8.94     35.8   62.6   89.4
     6       38     30     10.4     41.6   72.9   104.1

  Idle packets are 0xFF 0x00 0xFF
  with preamble and start bits this makes:
  (....11111111111111) 0 11111111 0 00000000 0 11111111 (1)
  
  

*/


#include <stdio.h>

#include "gertbot.h"


//
// Switch DCC set-up window on/off
// 
void gertbot::dccsetup_slot()
{ QObject *qto = (QObject *)sender();
  int b,m;
  QString str;
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  str.sprintf("DCC M=%d B=%d",m,b);
  gertbot_dccsetup dccsetup(this,0);
  dccsetup.ui.group->setTitle(str);
  dccsetup.set_dcc(&(dcc_gui[b][m]),&(dcc_gui[0][0]));
  if (dccsetup.exec())
  { dccsetup.get_dcc(&(dcc_gui[b][m]));
    set_dcc_gui(b,m,&(dcc_gui[b][m]));
  }

} // log on/off 


void gertbot::set_dcc_gui(int b,int m,dccsetup_type *setup)
{ Ui::motor *mtptr;
  int offset;
  mtptr = motor_gui[b][m].uipnt;
  if (setup->cmnd_length[0]==0) mtptr->pb_dcc0->setEnabled(false); else { mtptr->pb_dcc0->setEnabled(true); mtptr->pb_dcc0->setText(setup->pb_text[0]); }
  if (setup->cmnd_length[1]==0) mtptr->pb_dcc1->setEnabled(false); else { mtptr->pb_dcc1->setEnabled(true); mtptr->pb_dcc1->setText(setup->pb_text[1]); }
  if (setup->cmnd_length[2]==0) mtptr->pb_dcc2->setEnabled(false); else { mtptr->pb_dcc2->setEnabled(true); mtptr->pb_dcc2->setText(setup->pb_text[2]); }
  if (setup->cmnd_length[3]==0) mtptr->pb_dcc3->setEnabled(false); else { mtptr->pb_dcc3->setEnabled(true); mtptr->pb_dcc3->setText(setup->pb_text[3]); }
  if (setup->cmnd_length[4]==0) mtptr->pb_dcc4->setEnabled(false); else { mtptr->pb_dcc4->setEnabled(true); mtptr->pb_dcc4->setText(setup->pb_text[4]); }
  if (setup->cmnd_length[5]==0) mtptr->pb_dcc5->setEnabled(false); else { mtptr->pb_dcc5->setEnabled(true); mtptr->pb_dcc5->setText(setup->pb_text[5]); }
  if (setup->cmnd_length[6]==0) mtptr->pb_dcc6->setEnabled(false); else { mtptr->pb_dcc6->setEnabled(true); mtptr->pb_dcc6->setText(setup->pb_text[6]); }
  if (setup->cmnd_length[7]==0) mtptr->pb_dcc7->setEnabled(false); else { mtptr->pb_dcc7->setEnabled(true); mtptr->pb_dcc7->setText(setup->pb_text[7]); }
  if (setup->cmnd_length[8]==0) mtptr->pb_dcc8->setEnabled(false); else { mtptr->pb_dcc8->setEnabled(true); mtptr->pb_dcc8->setText(setup->pb_text[8]); }
  if (setup->cmnd_length[9]==0) mtptr->pb_dcc9->setEnabled(false); else { mtptr->pb_dcc9->setEnabled(true); mtptr->pb_dcc9->setText(setup->pb_text[9]); }
  if (setup->cmnd_length[10]==0)
  { mtptr->slide_dcc->setEnabled(false);
    mtptr->lbl_slide->setEnabled(false);
  }
  else
  {
    mtptr->slide_dcc->setEnabled(true);
    mtptr->lbl_slide->setEnabled(true);
    mtptr->lbl_slide->setText(setup->pb_text[10]);
    // For motor control want to have a decent 'dead' value in the middle
    // So the engine is really stopped and not 'crawling'
    if (setup->slider_number_data=='S')
    { if (setup->slider_min==14 || setup->slider_min==-14)
        offset = 1;
      else
        offset = 2;
    }
    else
      offset = 0;

    if (setup->slider_min>setup->slider_max)
    {
      mtptr->slide_dcc->setMinimum(setup->slider_max-offset);
      mtptr->slide_dcc->setMaximum(setup->slider_min+offset);
    }
    else
    {
      mtptr->slide_dcc->setMinimum(setup->slider_min-offset);
      mtptr->slide_dcc->setMaximum(setup->slider_max+offset);
    }
  }
  mtptr->slide_dcc->setPageStep(1);
  mtptr->slide_dcc->setTickPosition(QSlider::TicksBelow);
  mtptr->slide_dcc->setTickInterval(10);

} // set_dcc_gui  


void gertbot::dcc_init()
{ int m,b,p;
  for (b=0; b<4; b++)
  { for (m=0; m<4; m++)
    {
      dcc_gui[b][m].motor = m;
      dcc_gui[b][m].board = b;
      dcc_gui[b][m].name  = 0;
      for (p=0; p<11; p++)
      { dcc_gui[b][m].destination[p]=0;
        dcc_gui[b][m].pb_text[p][0]=0;
        dcc_gui[b][m].pb_command[p][0]=0;
        dcc_gui[b][m].cmnd_length[p] = 0;
      }
    }
  } 
} // dcc_init

//
// DCC command button pressed
//
void gertbot::dcc_but_slot()  
{ byte command[16];
  QObject *qto = (QObject *)sender();
  int b,m,p,l,i;
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  p = qto->objectName()[2].digitValue();
  assert(b>=0 && m>=0 && p>=0);
  l = dcc_gui[b][m].cmnd_length[p];
  if (l<0)
    return;
  if (l>5)
    l=5;
  command[0] = CMD_DCC_MESS;
  command[1] = b<<2;
  command[2] = (dcc_gui[b][m].destination[p]<<4) | l;
  for (i=0; i<l; i++)
    command[i+3]=dcc_gui[b][m].command[p][i];
  for ( ; i<5; i++)
    command[i+3]=0;
  uart_tx(command,8,1);
} // dcc_but_slot

//
// DCC slider operated
// A slider can produce many events in a short time
// that could easily overflow the message queue
// So here I have to take precautions
//
// slider does not give message but starts a timer.
// If new event arrives before timer expires restart timer.
// When timer times-out, take slider value and send it
// As the system is set-up now we can have 4 sliders
// so four simultaneous events.
// But if we use a short time-out value then timers will expires before
// a user can active a different slider.
// e.g. 1/10 of a second requires the user to release and active a second
// slider in 1/10 of a second
// 
void gertbot::dcc_slide_slot()
{ QObject *qto = (QObject *)sender();
  int b,m;
  b = slider_data[0] = qto->objectName()[0].digitValue();
  m = slider_data[1] = qto->objectName()[1].digitValue();
  if (dcc_gui[b][m].cmnd_length[10]<=0)
    return; // should not happen 
  dcc_timer.start(DCC_TIMEOUT);

} // dcc_slide_slot

//
// Slider (speed) tables
//
static int slide_28[29] = { 
    0x40, 
    0x42,0x52,    0x43,0x53,
    0x44,0x54,    0x45,0x55,
    0x46,0x56,    0x47,0x57,
    0x48,0x58,    0x49,0x59,
    0x4A,0x5A,    0x4B,0x5B,
    0x4C,0x5C,    0x4D,0x5D,
    0x4E,0x5E,    0x4F,0x5F
};

static int slide_14[15] = { 
    0x40, 
    0x42,    0x43,
    0x44,    0x45,
    0x46,    0x47,
    0x48,    0x49,
    0x4A,    0x4B,
    0x4C,    0x4D,
    0x4E,    0x4F
};

int find_code = 0x8000;
void gertbot::dcc_timer_slot()
{ byte command[16];
  int b,m,i,l,s;
  int byte,min,max;
  int value;
  b = slider_data[0];
  m = slider_data[1];
  l = dcc_gui[b][m].cmnd_length[10]; 

  s = dcc_gui[b][m].slider_number_pos;
  byte = 0;
  value = motor_gui[b][m].uipnt->slide_dcc->value();
  if (value==0)
    return;
  if (value==-1)
  { find_code = 0x8000;
    return;
  }
  min = dcc_gui[b][m].slider_min;
  max = dcc_gui[b][m].slider_max;
  if (min>max)
  { // reverse value from direction 
    value = (max+min)-value;
  }

///////////////////////////////

  command[0] = CMD_DCC_MESS;
  command[1] = b<<2;
  command[2] = 0xF2;
  command[3] = (find_code>>8) & 0xFF;
  command[4] = find_code & 0xFF;
  find_code = find_code + 1;
  uart_tx(command,8,1);
  dcc_timer.start(DCC_TIMEOUT*100);

  /****
  // Output bytes before value 
  // position is s so should have s/2 bytes before 
  for (i=0; i<(s>>1); i++)
    command[i+3]=dcc_gui[b][m].command[10][byte++];

  // Output value bytes
  switch(dcc_gui[b][m].slider_number_data)
  { 
  case '<':
    // Output digital value LS to MS
    while (dcc_gui[b][m].pb_command[10][s]=='<')
    { 
      command[i+3] = value & 0xFF;
      value>>=8;
      i++;
      s++;
      l++;
    }
    break;
  case '>' :
    // Output digital value MS to LS
    // Count how big 
    while (dcc_gui[b][m].pb_command[10][s]=='>')
    { byte++;
      s++;
    }
    while (byte--)
    {
      command[i+3] = (value>>(8*byte)) & 0xFF;
      i++;
      s++;
      l++;
    }
    break;
  case 'S':
    // Output speed control value 
    if (value<0)
    { byte = 0x20; // direction bit
      value = -value;
    }
    else
      byte = 0x00;
    if (min==14 || min==-14)
    { // compensate for dead zone 
      if (value<=1)
        byte |= slide_14[0];
      else
        byte |= slide_14[value-1];
    }
    if (min==28 || min==-28)
    { // compensate for dead zone 
      if (value<=2)
        byte |= slide_28[0];
      else
        byte |= slide_28[value-2];    
    }
    command[i+3] = byte;
    i++;
    s++;
    l++;
  } // switch slider data format 

  // Now we know message length 
  command[2] = (dcc_gui[b][m].destination[10]<<4) | l;

  // Output bytes after value
  for (  ; i<l; i++)
    command[i+3]=dcc_gui[b][m].command[10][byte++];

  // Output unused bytes 
  for ( ; i<5; i++)
    command[i+3]=0;
  uart_tx(command,8,1);
  ***/
} // dcc_timer_slot


void gertbot::dcc_config_slot()
{ byte cmd[8];
  QObject *qto = (QObject *)sender();
  int b,m;
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  cmd[0]=CMD_DCC_CONFIG;
  cmd[1]= (b<<2) | m;
  cmd[2] = motor_gui[b][m].uipnt->sb_dcc_repeat->value();
  cmd[3] = motor_gui[b][m].uipnt->sb_dcc_preamble->value();
  cmd[4] = motor_gui[b][m].uipnt->sb_dcc_dc->value();
  cmd[5] = motor_gui[b][m].uipnt->cb_dcc_idle_off->isChecked() ? 0x01 : 0x00;
  uart_tx(cmd,6,0);
} // dcc_config_slot


