//
// Gertbot GUI
//
//

#include "gertbot_dccsetup.h"

gertbot_dccsetup::gertbot_dccsetup(QWidget *parent, Qt::WFlags flags)
    : QDialog(parent, flags)
{ 

  last_dcc_dir = ".";
  dcc_filename = "";
  help_screen = false;

  ui.setupUi(this);
  // Add buttons to button group 
  ub.addButton(ui.pb_but0,0);
  ub.addButton(ui.pb_but1,1);
  ub.addButton(ui.pb_but2,2);
  ub.addButton(ui.pb_but3,3);
  ub.addButton(ui.pb_but4,4);
  ub.addButton(ui.pb_but5,5);
  ub.addButton(ui.pb_but6,6);
  ub.addButton(ui.pb_but7,7);
  ub.addButton(ui.pb_but8,8);
  ub.addButton(ui.pb_but9,9);
  ub.addButton(ui.pb_slide,10);
  // Hide min/max 
  ui.sb_min->hide();
  ui.sb_max->hide();
  ui.sl_min->hide();
  ui.sl_max->hide();

  // pb[10]
//  connect(this  ,SIGNAL(finished(int)), parent, SLOT(log_close_slot()));
  connect(ui.pd_file  ,SIGNAL(activated(int)), this, SLOT(file_slot(int)));
  connect(&ub,SIGNAL(buttonClicked(int)), this, SLOT(pb_slot(int)));
  connect(ui.pb_done,SIGNAL(clicked(bool)), this, SLOT(done_slot()));
  connect(ui.pb_exit,SIGNAL(clicked(bool)), this, SLOT(reject()));
  connect(ui.pb_help,SIGNAL(clicked(bool)), this, SLOT(help_slot()));
  connect(ui.pb_help_back,SIGNAL(clicked(bool)), this, SLOT(help_slot()));

  cur_but = -1;
}

gertbot_dccsetup::~gertbot_dccsetup()
{
 
}
/*
 char *name;
 int  board;
 int  motor;
 char *pb_text[11];
 char *pb_command[11];
 int   cmnd_length[11];
 unsigned char *command[11];
 int   slider_number_pos;
 int   slider_number_data;
 int   slider_min,slider_max;
*/

void gertbot_dccsetup::set_dcc(dccsetup_type *dccs,dccsetup_type *dccs_pnt)
{ local_dcc = *dccs;
  parent_dcc = dccs_pnt;
  pb_slot(0);
}

void gertbot_dccsetup::get_dcc(dccsetup_type *dccs)
{ *dccs = local_dcc;
}

void gertbot_dccsetup::done_slot()
{ // before leaving store the current data
  pb_slot(-1);
  this->accept();
}


//
// Convert QString to command
// buffer must accept max 64 bytes
// return number of bytes converted
// return -1 on error
// *legal is the string of non-hex characters which are seen as 'legal'
int gertbot_dccsetup::dcc_str_to_bin(unsigned char *buffer,QString str,char *legal)
{ char asc,*lp;
  int n,i,hex,l,h;
  n=0;
  l = str.length();
  for (i=0; i<l && i<128; i++)
  {
    asc = str.at(i).cell();
    if (isxdigit(asc))
    { if (asc<='9')
        hex = asc-'0';
      else
      { if (asc<='F')
          hex = asc-'A'+10;
       else
          hex = asc-'a'+10;
      }
      if (n & 1)
        buffer[n>>1] |= hex;
      else
        buffer[n>>1] = hex<<4;
      n++;
    } // is xdigit
    else
    { // check legals
      h = 0;
      lp = legal;
      while (*lp)
      { if (asc==*lp++)
          h=1;
      }
      if (h==0)
        return 0;
    }
  } // for each char
  return n>>1;
} // dcc_str_to_bin

//
// Button pressed 
// Also used in general to store or load the GUI
//
//
void gertbot_dccsetup::pb_slot(int b)
{ int pos,type;
  
  // copy from controls into structure 
  if (cur_but>=0)
  { 
    strcpy(local_dcc.pb_text[cur_but],ui.le_but_text->text().toLatin1());
    strcpy(local_dcc.pb_command[cur_but],ui.le_command->text().toLatin1());
    local_dcc.destination[cur_but]  = ui.cb1->isChecked() ? 0x01 : 0x00;
    local_dcc.destination[cur_but] |= ui.cb2->isChecked() ? 0x02 : 0x00;
    local_dcc.destination[cur_but] |= ui.cb3->isChecked() ? 0x04 : 0x00;
    local_dcc.destination[cur_but] |= ui.cb4->isChecked() ? 0x08 : 0x00;

    if (cur_but!=10)
    { local_dcc.cmnd_length[cur_but] = 
        dcc_str_to_bin(local_dcc.command[cur_but],ui.le_command->text()," ");
      if (local_dcc.command[cur_but][0])
      { //  error checks when command is not empty
        if (local_dcc.cmnd_length[cur_but]==0)
        { message_box.setText("Syntax error in command");
          message_box.exec();       
        }
        if (local_dcc.cmnd_length[cur_but]<2 || local_dcc.cmnd_length[cur_but]>5)
        { 
           message_box.setText("Command length must be in range 2..5");
           message_box.exec();       
           local_dcc.cmnd_length[cur_but] = 0;
        }
      }
    }
    else
    { // process slider data
      local_dcc.cmnd_length[10] = 
        dcc_str_to_bin(local_dcc.command[10],ui.le_command->text()," ><S");
      if (local_dcc.cmnd_length[10]==0)
      {
        if (local_dcc.command[cur_but][0])
        { message_box.setText("Syntax error in command");
          message_box.exec();       
        }
      }
      local_dcc.slider_min = ui.sb_min->value();
      local_dcc.slider_max = ui.sb_max->value();
      type = pos = 0;
      if (ui.le_command->text().indexOf(">")) type = 1;
      if (ui.le_command->text().indexOf("<")) type = 2;
      if (ui.le_command->text().indexOf("S")) type = 3;
      switch (type)
      {
      case 0 : 
           message_box.setText("No value indicator ('>, < or S') found");
           message_box.exec();       
           local_dcc.cmnd_length[10] = 0;
           break;
      case 1 : 
           pos = ui.le_command->text().indexOf(">");
           local_dcc.slider_number_data = '>';
           break;
      case 2 : 
           pos = ui.le_command->text().indexOf("<");
           local_dcc.slider_number_data = '<';
           break;
      case 3 : 
           pos = ui.le_command->text().indexOf("S");
           local_dcc.slider_number_data = 'S';
           break;
      } // switch indicator type
      
      if (pos & 1)
      { 
         message_box.setText("Illegal hex before indicator ");
         message_box.exec();       
//         local_dcc.cmnd_length[10] = 0;
      }
      local_dcc.slider_number_pos = pos;

      if (local_dcc.cmnd_length[cur_but])
      { // if we have a length check some more stuff
        if (local_dcc.destination[cur_but]==0)
        {
          message_box.setText("Warning: No channels selected");
          message_box.exec();       
        }
      }

    }
  } // have current

  // new current button
  cur_but = b;
  if (cur_but>=0)
  {
    // Take data and fill controls
    ui.le_but_text->setText(local_dcc.pb_text[cur_but]);
    ui.le_command->setText(local_dcc.pb_command[cur_but]);
    ui.cb1->setChecked(local_dcc.destination[cur_but]& 0x01 ? true : false);
    ui.cb2->setChecked(local_dcc.destination[cur_but]& 0x02 ? true : false);
    ui.cb3->setChecked(local_dcc.destination[cur_but]& 0x04 ? true : false);
    ui.cb4->setChecked(local_dcc.destination[cur_but]& 0x08 ? true : false);
    if (cur_but==10)
    { ui.sb_min->show();
      ui.sb_max->show();
      ui.sb_min->setValue(local_dcc.slider_min);
      ui.sb_max->setValue(local_dcc.slider_max);
      ui.sl_min->show();
      ui.sl_max->show();
      ui.sl_butext->setText("Slide text");
    }
    else
    { ui.sb_min->hide();
      ui.sb_max->hide();
      ui.sl_min->hide();
      ui.sl_max->hide();
      ui.sl_butext->setText("Button text");
    }
  }

} // pb_slot

//
// Write DCC data to file
//
int gertbot_dccsetup::save_dcc(FILE *fp,dccsetup_type *dccs)
{ int p,i;
  QStringList lines;
  char text[128];
  
  fprintf(fp,"%d %d version\n",0,0);
  if (dccs->name)
    fprintf(fp,"%s\n",dccs->name);
  else
    fprintf(fp,"\n");
  // description 
  lines = ui.te_descr->toPlainText().split("\n");
  p = lines.count();
  fprintf(fp,"%d\n",p);
  for (i=0; i<p; i++)
  { strncpy(text,lines.at(i).toAscii(),128);
    fprintf(fp,"%s\n",text);
  }


  fprintf(fp,"%d %d brd,mot\n",dccs->board,dccs->motor);
  for (p=0; p<11; p++)
  { fprintf(fp,"%02X\n",dccs->destination[p]);
    fprintf(fp,"%s\n",dccs->pb_text[p]);
    fprintf(fp,"%s\n",dccs->pb_command[p]);
  }
  fprintf(fp,"%d %d\n",dccs->slider_min,dccs->slider_max);
  fprintf(fp,"%d %d\n",dccs->slider_number_pos,dccs->slider_number_data);
  return 0;
} // save_dcc

//
// Copy string stopping before CR LF 
//
void chomp(char *dest,const char *source)
{ 
  while (*source && *source!=0x0D && *source!=0x0A)
    *dest++ = *source++;
  *dest = 0;
}
//
// Read DCC data from file 
//
int gertbot_dccsetup::load_dcc(FILE *fp,dccsetup_type *dccs)
{ int p,l;
  char text[128];
  QString str;
  fgets(text,128,fp); // version: not yet used 
  if (feof(fp))
    return 0;
  fgets(text,128,fp); // dccs->name: not yet supported
  fgets(text,128,fp);   sscanf(text,"%d",&p);
  ui.te_descr->clear();
  while (p--)
  { fgets(text,128,fp);
    l = strlen(text)-1;
    // chomp
    while (l && (text[l]==0x0D || text[l]==0x0A))
      text[l--]=0;
    ui.te_descr->appendPlainText(text);
  }
  fgets(text,128,fp);   sscanf(text,"%d %d",&(dccs->board),&(dccs->motor));
  for (p=0; p<11; p++)
  { 
    fgets(text,128,fp);   sscanf(text,"%x",&(dccs->destination[p]));
    fgets(text,128,fp);   chomp(dccs->pb_text[p],text);
    fgets(text,128,fp);   chomp(dccs->pb_command[p],text);
    // Build digital command 
    if (p==10)
      dccs->cmnd_length[p] = 
        dcc_str_to_bin(dccs->command[p],dccs->pb_command[p]," <>S");
    else
      dccs->cmnd_length[p] = 
        dcc_str_to_bin(dccs->command[p],dccs->pb_command[p]," ");

  }
  fgets(text,128,fp);   sscanf(text,"%d %d",&(dccs->slider_min),&(dccs->slider_max));
  fgets(text,128,fp);   sscanf(text,"%d %d",&(dccs->slider_number_pos),&(dccs->slider_number_data));
  return 1;
} // load_dcc

//
// File slot 
//
// Todo: replace with my usual file open/close code which
// remembers the last directory, sets the default extension etc. 
//
void gertbot_dccsetup::file_slot(int mode)
{ FILE *fp;
  int p,m,b;
  QString fileName,str;
  char text[128];
  dccsetup_type *dcc_pnt;
  // Set control back to 'File'
  ui.pd_file->setCurrentIndex(0); 
  pb_slot(cur_but); // save state 
  switch (mode)
  {
  case 0 : // File
      break;
  case 1 : // save
      if (dcc_filename.isEmpty())
         fileName = QFileDialog::getSaveFileName(this,"Save DCC",last_dcc_dir,
           "DCC (*.dcc);;All (*.*)");
      if (fileName.isEmpty())
        return; // user gave no name: no saving!
      fp = fopen(fileName.toLatin1(),"w");
      if (!fp)
      {  message_box.setText("Could not open '" + fileName + "' for writing");
         message_box.exec();       
        return;
      }
      save_dcc(fp,&local_dcc);
      fclose(fp);
      break;
  case 2 : // save as
      fileName = QFileDialog::getSaveFileName(this,"Save DCC",last_dcc_dir,
           "DCC (*.dcc);;All (*.*)");
      if (fileName.isEmpty())
        return; // user gave no name: no saving!
      fp = fopen(fileName.toLatin1(),"w");
      if (!fp)
      {  message_box.setText("Could not open '" + fileName + "' for writing");
          message_box.exec();       
         return;
      }
      save_dcc(fp,&local_dcc);
      fclose(fp);
      break;
  case 3 : // Load
      fileName = QFileDialog::getOpenFileName(
        this,"Load DCC",last_dcc_dir,
           "DCC (*.dcc);;All (*.*)");
      if (fileName.isEmpty())
        return; // user gave no name: no saving!
      fp = fopen(fileName.toLatin1(),"r");
      if (!fp)
      {  message_box.setText("Could not open '" + fileName + "' for reading");
         message_box.exec();       
        return;
      }
      if (!load_dcc(fp,&local_dcc))
      {  message_box.setText("Error while reading '" + fileName + "'");
         message_box.exec();       
      }
      fclose(fp);
      // Set gui from new loaded data 
      p = cur_but;
      cur_but = -1; // invalid 
      pb_slot(p); // set gui from current 
      break;

  case 4 : // Load all
      fileName = QFileDialog::getOpenFileName(
        this,"Load DCC",last_dcc_dir,
           "DCC (*.dca);;All (*.*)");
      if (fileName.isEmpty())
        return; // user gave no name: no saving!
      fp = fopen(fileName.toLatin1(),"r");
      if (!fp)
      {  message_box.setText("Could not open '" + fileName + "' for reading");
         message_box.exec();       
        return;
      }
      // check the file type
      fgets(text,128,fp);
      if (strncmp(text,"DCCA",4))
      { message_box.setText("File is not a DCC-all file");
        message_box.exec();       
        fclose(fp);
        return;
      }
      // This is dirty
      for (b=0; b<4; b++)
      { for (m=0; m<4; m++)
        { dcc_pnt = parent_dcc +b*4+m;
          if (!load_dcc(fp,dcc_pnt))
          {  str.sprintf("Error loading data board:%d motor:%d",b,m);
             message_box.setText(str);
             message_box.exec();
             fclose(fp);
             return;
          }
        }
      }
      // copy local from  parent
      fclose(fp);
      // Now parent data has been re-loaded but we still have a local copy here 
      b=local_dcc.board;
      m=local_dcc.motor;
      local_dcc = *(parent_dcc + b*4 + m); 
      // Set gui from new loaded data 
      p = cur_but;
      cur_but = -1; // invalid 
      pb_slot(p); // set gui from current 
      break;

  case 5 : // Save all
  case 6 : // Save all as
      fileName = QFileDialog::getSaveFileName(this,"Save DCC",last_dcc_dir,
           "DCC (*.dca);;All (*.*)");
      if (fileName.isEmpty())
        return; // user gave no name: no saving!
      fp = fopen(fileName.toLatin1(),"w");
      if (!fp)
      { message_box.setText("Could not open '" + fileName + "' for writing");
        message_box.exec();       
        return;
      }
      // Set DCC-all type 
      fprintf(fp,"DCCA\n");
      // Dirty 
      for (b=0; b<4; b++)
        for (m=0; m<4; m++)
        { // When saving parent, slot in the local data
          if (b==local_dcc.board && m==local_dcc.motor)
            dcc_pnt = &local_dcc;
          else
            dcc_pnt = parent_dcc +b*4+m;
          save_dcc(fp,dcc_pnt);
        }
       fclose(fp);
      break;
  }
} // file_slot

void gertbot_dccsetup::help_slot()
{
  if (help_screen)
  {
    ui.stack->setCurrentIndex(0);
    help_screen = false;
  }
  else
  {
    ui.stack->setCurrentIndex(1);
     help_screen = true;
  }
}

