//
// Gertbot GUI
//
// Building and updating the Graphical User Interface
//
//
#include <stdio.h>

#include "gertbot.h"

extern bool uart_opened;

//
// Make GUI for one motor
// Startup in DC/brushed mode 
// Usding Qt-designer prepared widget
//
bool gertbot::make_motor_group(int board,int mot)
{ Ui::motor *mtpnt;  // motor group box
  motor_gui_type *mgpnt;
  char       my_index[4];
  int        x;

  mgpnt = &(motor_gui[board][mot]);
  if (mgpnt->uipnt!=NULL)
    return false; // already done
  // Tell me who I am 
  // (in case I am passed as pointer) 
  mgpnt->my_b = board;
  mgpnt->my_m = mot;
  mgpnt->more_index = 0;

  // Instance module

  // Build frame to put UI in 
  motor_gui[board][mot].motor_frame = new QFrame(tab[board]);
  x = mot*162+1;
  mgpnt->motor_frame->setGeometry(QRect(  x, 1,  170,  200));
  mgpnt->motor_frame->setFrameShape(QFrame::NoFrame);
  mgpnt->uipnt = new (Ui::motor);
  mgpnt->uipnt->setupUi(mgpnt->motor_frame);

  // Set-up the variable parts of the GUI 
  mtpnt = mgpnt->uipnt;
  switch(mot)
  {
  case 0 : 
    mtpnt->gb_motor->setTitle("Motor 0");
    mtpnt->pd_short->insertItem(1,"Stop 0");
    mtpnt->pd_short->insertItem(2,"Stop 0&1");
    break;
  case 1 : 
    mtpnt->gb_motor->setTitle("Motor 1");
    mtpnt->pd_short->insertItem(1,"Stop 1");
    mtpnt->pd_short->insertItem(2,"Stop 0&1");
    break;
  case 2 : 
    mtpnt->gb_motor->setTitle("Motor 2");
    mtpnt->pd_short->insertItem(1,"Stop 2");
    mtpnt->pd_short->insertItem(2,"Stop 2&3");
    break;
  case 3 : 
    mtpnt->gb_motor->setTitle("Motor 3");
    mtpnt->pd_short->insertItem(1,"Stop 3");
    mtpnt->pd_short->insertItem(2,"Stop 2&3");
    break;
  }
  mtpnt->pd_short->insertItem(0,"None");
  mtpnt->pd_short->insertItem(3,"Stop 0-3");
  mtpnt->pd_short->insertItem(4,"Stop All");
  
  // Todo : Set icons in designer 
  mtpnt->pb_brushA->setIcon(icon_left); 
  mtpnt->pb_brushS->setIcon(icon_pause);
  mtpnt->pb_brushB->setIcon(icon_right);
  mtpnt->pb_stepA->setIcon(icon_left); 
  mtpnt->pb_stepS->setIcon(icon_pause);
  mtpnt->pb_stepB->setIcon(icon_right);
  mtpnt->pb_stepI->setIcon(icon_stop);

  // text : (No support for translation)
  mtpnt->pd_mode->insertItem(0,"Off");
  mtpnt->pd_mode->insertItem(1,"DC/brushed");
  mtpnt->pd_mode->insertItem(2,"Dig. Cmnd Ctrl");
  if (!(mot & 1))
  { // Only even motors can select stepper modes 
    mtpnt->pd_mode->insertItem(3,"Step Gray  Off");
    mtpnt->pd_mode->insertItem(4,"Step Pulse Off");
    mtpnt->pd_mode->insertItem(5,"Step Gray  Pwr");
    mtpnt->pd_mode->insertItem(6,"Step Pulse Pwr");
  }

  // NASTY!!
  // I want to know which board/motor the object belongs to
  // So I am mis-using QT's objectname 
  // Make string "xy" x=board, y=motor 
  my_index[0] = board+'0';
  my_index[1] = mot+'0';
  my_index[2] = 0;
  mtpnt->pd_mode->setObjectName(my_index);
  mtpnt->pb_more->setObjectName(my_index);
  mtpnt->le_brushfreq->setObjectName(my_index);
  mtpnt->sb_dutycycle->setObjectName(my_index);
  mtpnt->pb_brushA->setObjectName(my_index);
  mtpnt->pb_brushS->setObjectName(my_index);
  mtpnt->pb_brushB->setObjectName(my_index);
  mtpnt->pb_stepA->setObjectName(my_index);
  mtpnt->pb_stepS->setObjectName(my_index);
  mtpnt->pb_stepB->setObjectName(my_index);
  mtpnt->pb_stepI->setObjectName(my_index);
  mtpnt->le_stepfreq->setObjectName(my_index);
  mtpnt->le_steps->setObjectName(my_index);
  mtpnt->pd_short->setObjectName(my_index);
  mtpnt->pb_more->setObjectName(my_index);
  mtpnt->pb_dcc_setup->setObjectName(my_index);
  mtpnt->pd_ramp_up->setObjectName(my_index); 
  mtpnt->pd_ramp_down->setObjectName(my_index); 
  mtpnt->pd_ramp_halt->setObjectName(my_index); 
  

  mtpnt->sb_dcc_repeat->setObjectName(my_index);
  mtpnt->sb_dcc_repeat->setObjectName(my_index);
  mtpnt->sb_dcc_repeat->setObjectName(my_index);
  mtpnt->cb_dcc_idle_off->setObjectName(my_index);

  // End stop controls have extra, third number 
  my_index[3] = 0;
  my_index[2] = '0';   mtpnt->cb_endstop_A->setObjectName(my_index); 
  my_index[2] = '1';   mtpnt->cb_endstop_B->setObjectName(my_index); 
  my_index[2] = '2';   mtpnt->cb_high_A   ->setObjectName(my_index); 
  my_index[2] = '3';   mtpnt->cb_high_B   ->setObjectName(my_index); 
  // As do the DCC buttons
  my_index[2] = '0'; mtpnt->pb_dcc0->setObjectName(my_index); 
  my_index[2] = '1'; mtpnt->pb_dcc1->setObjectName(my_index); 
  my_index[2] = '2'; mtpnt->pb_dcc2->setObjectName(my_index); 
  my_index[2] = '3'; mtpnt->pb_dcc3->setObjectName(my_index); 
  my_index[2] = '4'; mtpnt->pb_dcc4->setObjectName(my_index); 
  my_index[2] = '5'; mtpnt->pb_dcc5->setObjectName(my_index); 
  my_index[2] = '6'; mtpnt->pb_dcc6->setObjectName(my_index); 
  my_index[2] = '7'; mtpnt->pb_dcc7->setObjectName(my_index); 
  my_index[2] = '8'; mtpnt->pb_dcc8->setObjectName(my_index); 
  my_index[2] = '9'; mtpnt->pb_dcc9->setObjectName(my_index); 
  my_index[2] = 'A'; mtpnt->slide_dcc->setObjectName(my_index); 
  
  my_index[2] = '1'; mtpnt->sb_dcc_preamble->setObjectName(my_index); 
  my_index[2] = '2'; mtpnt->sb_dcc_dc->setObjectName(my_index); 


 
  connect(mtpnt->pd_mode, SIGNAL(activated(int)), this, SLOT(pd_mode_slot()));
  connect(mtpnt->cb_endstop_A,SIGNAL(stateChanged(int)), this, SLOT(endmode_slot()));
  connect(mtpnt->cb_endstop_B,SIGNAL(stateChanged(int)), this, SLOT(endmode_slot()));
  connect(mtpnt->cb_high_A,SIGNAL(stateChanged(int)), this, SLOT(endmode_slot()));
  connect(mtpnt->cb_high_B,SIGNAL(stateChanged(int)), this, SLOT(endmode_slot()));
  connect(mtpnt->le_brushfreq,SIGNAL(editingFinished()), this, SLOT(freq_slot()));
  connect(mtpnt->le_stepfreq,SIGNAL(editingFinished()), this, SLOT(freq_slot()));
  connect(mtpnt->le_steps,SIGNAL(editingFinished()), this, SLOT(steps_slot()));
  connect(mtpnt->sb_dutycycle,SIGNAL(valueChanged(double)), this, SLOT(dutycycle_slot()));
  connect(mtpnt->pb_brushA,SIGNAL(clicked(bool)), this, SLOT(runA_slot()));
  connect(mtpnt->pb_brushS,SIGNAL(clicked(bool)), this, SLOT(runS_slot()));
  connect(mtpnt->pb_brushB,SIGNAL(clicked(bool)), this, SLOT(runB_slot()));
  connect(mtpnt->pb_stepA,SIGNAL(clicked(bool)), this, SLOT(runA_slot()));
  connect(mtpnt->pb_stepS,SIGNAL(clicked(bool)), this, SLOT(runS_slot()));
  connect(mtpnt->pb_stepB,SIGNAL(clicked(bool)), this, SLOT(runB_slot()));
  connect(mtpnt->pb_stepI,SIGNAL(clicked(bool)), this, SLOT(runI_slot()));
  connect(mtpnt->pd_short,SIGNAL(activated(int)), this, SLOT(short_slot(int)));
  connect(mtpnt->pb_more,SIGNAL(clicked(bool)), this, SLOT(more_slot()));
  connect(mtpnt->pd_ramp_up, SIGNAL(activated(int)), this, SLOT(ramp_slot()));
  connect(mtpnt->pd_ramp_down, SIGNAL(activated(int)), this, SLOT(ramp_slot()));
  connect(mtpnt->pd_ramp_halt, SIGNAL(activated(int)), this, SLOT(ramp_slot()));

  connect(mtpnt->pb_dcc_setup,SIGNAL(clicked(bool)), this, SLOT(dccsetup_slot()));

  connect(mtpnt->pb_dcc0,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc1,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc2,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc3,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc4,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc5,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc6,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc7,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc8,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->pb_dcc9,SIGNAL(clicked(bool)), this, SLOT(dcc_but_slot()));
  connect(mtpnt->slide_dcc,SIGNAL(valueChanged(int)), this, SLOT(dcc_slide_slot()));

  connect(mtpnt->sb_dcc_repeat,SIGNAL(valueChanged(int)), this, SLOT(dcc_config_slot()));
  connect(mtpnt->sb_dcc_preamble,SIGNAL(valueChanged(int)), this, SLOT(dcc_config_slot()));
  connect(mtpnt->sb_dcc_dc,SIGNAL(valueChanged(int)), this, SLOT(dcc_config_slot()));
  connect(mtpnt->cb_dcc_idle_off,SIGNAL(stateChanged(int)), this, SLOT(dcc_config_slot()));


  return true;
} // make_motor_group

void gertbot::make_motor_switch(int b)
{ char my_index[3];
  my_index[0] = b+'0';
  my_index[1] = 0;
  my_index[2] = 0;
  board_gui[b].pb_motor_to_board = new QPushButton("Board",tab[b]);
  board_gui[b].pb_motor_to_board->setObjectName(my_index);
  board_gui[b].pb_motor_to_board->setGeometry(QRect(  510, 210,  75,  25));
  board_gui[b].pb_motor_to_board->setToolTip("Press to switch between motor controls and board controls");
  connect(board_gui[b].pb_motor_to_board,SIGNAL(clicked(bool)), this, SLOT(board_mot_slot()));
  board_gui[b].pb_motor_to_board->show();
} // make_motor_switch

#define CBOXY 55
#define CBOXH 45
#define CBOXX 42
#define CBOXW 47

//
// Build widgets to control board
//
// I use a pre-designed widget derived from board.ui
// Then I fill in the arrays of pointers with the values from that 
//
// TODO: do the same with motor controls....
void gertbot::make_board_gui(int b)
{ board_gui_type *bgpnt;
//  QWidget *wp;
  QString str;
  int p;
  char my_index[3];

  // The index is a 1 or 2 character string (plus 0) which holds 
  // the board number and the number of the control both in ASCII
  // The control name is set to this string and later the ASCII
  // values are converted to numbers to find out which board/motor the command came from 
  my_index[0] = b+'0'; 
  my_index[1] = 0;
  my_index[2] = 0;

  bgpnt = &(board_gui[b]);
  if (bgpnt->pb_board_to_motor!=NULL)
    return; // already built


  bgpnt->j3_frame    = new QFrame(tab[b]);
  bgpnt->j3_frame->setGeometry(QRect(  1, 1,  500,  20));
  bgpnt->j3_frame->setFrameShape(QFrame::NoFrame);
  bgpnt->j3_frame->hide();
 
  boardui[b].setupUi(bgpnt->j3_frame);
  bgpnt->pb_board_to_motor = boardui[b].pb_motors;
  // convert gui individual controls to array   
  j3_gui[b].pd[0] = boardui[b].pd00;
  j3_gui[b].pd[1] = boardui[b].pd01;
  j3_gui[b].pd[2] = boardui[b].pd02;
  j3_gui[b].pd[3] = boardui[b].pd03;
  j3_gui[b].pd[4] = boardui[b].pd04;
  j3_gui[b].pd[5] = boardui[b].pd05;
  j3_gui[b].pd[6] = boardui[b].pd06;
  j3_gui[b].pd[7] = boardui[b].pd07;
  j3_gui[b].pd[8] = boardui[b].pd08;
  j3_gui[b].pd[9] = boardui[b].pd09;
  j3_gui[b].pd[10] = boardui[b].pd010;
  j3_gui[b].pd[11] = boardui[b].pd011;
  j3_gui[b].pd[12] = boardui[b].pd012;
  j3_gui[b].pd[13] = boardui[b].pd013;
  j3_gui[b].pd[14] = boardui[b].pd014;
  j3_gui[b].pd[15] = boardui[b].pd015;
  
  j3_gui[b].cb[0] = boardui[b].cb00;
  j3_gui[b].cb[1] = boardui[b].cb01;
  j3_gui[b].cb[2] = boardui[b].cb02;
  j3_gui[b].cb[3] = boardui[b].cb03;
  j3_gui[b].cb[4] = boardui[b].cb04;
  j3_gui[b].cb[5] = boardui[b].cb05;
  j3_gui[b].cb[6] = boardui[b].cb06;
  j3_gui[b].cb[7] = boardui[b].cb07;
  j3_gui[b].cb[8] = boardui[b].cb08;
  j3_gui[b].cb[9] = boardui[b].cb09;
  j3_gui[b].cb[10] = boardui[b].cb010;
  j3_gui[b].cb[11] = boardui[b].cb011;
  j3_gui[b].cb[12] = boardui[b].cb012;
  j3_gui[b].cb[13] = boardui[b].cb013;
  j3_gui[b].cb[14] = boardui[b].cb014;
  j3_gui[b].cb[15] = boardui[b].cb015;

  j3_gui[b].sb_dac[0] = boardui[b].sb_dac0;
  j3_gui[b].sb_dac[1] = boardui[b].sb_dac1;
#ifdef _WIN32
#else
  // Linux font too big for three digits so:
  j3_gui[b].sb_dac[0]->setDecimals(2);
  j3_gui[b].sb_dac[1]->setDecimals(2);
#endif

  j3_gui[b].lbl_adc[0]= boardui[b].lbl_adc0;
  j3_gui[b].lbl_adc[1]= boardui[b].lbl_adc1;
  j3_gui[b].lbl_adc[2]= boardui[b].lbl_adc2;
  j3_gui[b].lbl_adc[3]= boardui[b].lbl_adc3;

  // Give each widget object name the board & pin index 
  for (p=0; p<16; p++)
  { my_index[1] = '0'+p;
    j3_gui[b].pd[p]->setObjectName(my_index); 
    connect(j3_gui[b].pd[p],SIGNAL(activated(int)), this, SLOT(j3pd_slot()));
    j3_gui[b].cb[p]->setObjectName(my_index);
    connect(j3_gui[b].cb[p], SIGNAL(stateChanged(int)), this, SLOT(j3cb_slot(int)));    
  }

  //
  // ADC
  // 
  // At startup ADC are enabled
  // Hide ADC check boxes
  j3_gui[b].cb[10]->hide();
  j3_gui[b].cb[11]->hide();
  j3_gui[b].cb[12]->hide();
  j3_gui[b].cb[13]->hide();


  //
  // DAC 
  //
  j3_gui[b].sb_dac[0]->setMaximum(DAC_MAX);
  j3_gui[b].sb_dac[0]->setMinimum(DAC_MIN);
  my_index[1] = '0';
  j3_gui[b].sb_dac[0]->setObjectName(my_index);
  connect(j3_gui[b].sb_dac[0], SIGNAL(valueChanged(double)), this, SLOT(dac_value_slot()));

  j3_gui[b].sb_dac[1]->setMaximum(DAC_MAX);
  j3_gui[b].sb_dac[1]->setMinimum(DAC_MIN);
  my_index[1] = '1';
  j3_gui[b].sb_dac[1]->setObjectName(my_index);
  connect(j3_gui[b].sb_dac[1], SIGNAL(valueChanged(double)), this, SLOT(dac_value_slot()));
  // At start up DAC pins are inputs. Hide spinboxes
  j3_gui[b].sb_dac[0]->hide();
  j3_gui[b].sb_dac[1]->hide();

  // Read 
  boardui[b].pb_read->setObjectName(my_index);
  connect(boardui[b].pb_read,SIGNAL(clicked(bool)), this, SLOT(board_read_slot()));
  // Pin 
  boardui[b].pb_disp->setObjectName(my_index);
  connect(boardui[b].pb_disp,SIGNAL(clicked()), this, SLOT(board_pin_slot()));

  bgpnt->misc_group  = boardui[b].misc_group;

  // Misc: Open drains and sync mode 
  connect(boardui[b].cb_syncmode,SIGNAL(stateChanged(int)), this, SLOT(sync_mode_slot(int)));
  my_index[1] = '0';
  boardui[b].cb_opendrain0->setObjectName(my_index);
  connect(boardui[b].cb_opendrain0,SIGNAL(stateChanged(int)), this, SLOT(opendrain_slot()));
  my_index[1] = '1';
  boardui[b].cb_opendrain1->setObjectName(my_index);
  connect(boardui[b].cb_opendrain1,SIGNAL(stateChanged(int)), this, SLOT(opendrain_slot()));
  boardui[b].cb_syncmode->setObjectName(my_index);

  // Attention mode
   boardui[b].pd_attention->setObjectName(my_index);
  connect(boardui[b].pd_attention,SIGNAL(activated(int)), this, SLOT(attention_slot(int)));

  // Board to motor 
  boardui[b].pb_motors->setObjectName(my_index);
  connect(boardui[b].pb_motors,SIGNAL(clicked(bool)), this, SLOT(board_mot_slot()));

} // make_board_gui


//
// Adapt gui when switching modes 
//
void gertbot::set_motormode_gui(int board,int mot)
{ Ui::motor *mtptr; // 
  int mode;
  mtptr = motor_gui[board][mot].uipnt;
  mode = index_to_mode[ mtptr->pd_mode->currentIndex() ];
  
  switch (mode)
  {
  case MOT_MODE_OFF : // off
    mtptr->mode_stack->setCurrentIndex(MOT_PAGE_OFF);
    // Need to enable the odd channels again? 
    if ((mot==0 || mot==2) && !motor_gui[board][mot+1].uipnt->gb_motor->isEnabled())
        motor_gui[board][mot+1].uipnt->gb_motor->setEnabled(true);
    // off has no 'more'
    mtptr->pb_more->setEnabled(false);
    break;

  case MOT_MODE_BRUSH : // DC/brushed
    switch (motor_gui[board][mot].more_index)
    {
    case 0 : 
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_BRUSH);
      mtptr->pb_more->setText("More");
      break;
    case 1 : 
      mtptr->group_ramp->setEnabled(true);
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_SET);
      mtptr->pb_more->setText("Back");
      break;
    }
    // May need to enable odd channel 
    if ((mot==0 || mot==2) && !motor_gui[board][mot+1].uipnt->gb_motor->isEnabled())
        motor_gui[board][mot+1].uipnt->gb_motor->setEnabled(true);
    // Brushed has  'more'
    mtptr->pb_more->setEnabled(true);
    break;

  case MOT_MODE_STEP_GRAY:
  case MOT_MODE_STEP_PULSE:
  case (MOT_MODE_STEP_GRAY|MOT_MODE_STEP_HOLD):
  case (MOT_MODE_STEP_PULSE|MOT_MODE_STEP_HOLD):
    switch (motor_gui[board][mot].more_index)
    {
    case 0 : 
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_STEP);
      mtptr->pb_more->setText("More");
      break;
    case 1 : 
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_SET);
      mtptr->group_ramp->setEnabled(false);
      mtptr->pb_more->setText("Back");
      break;
    }
    // switch off the odd channel
    motor_gui[board][mot+1].uipnt->pd_mode->setCurrentIndex(0); // off 
    motor_gui[board][mot+1].uipnt->mode_stack->setCurrentIndex(MOT_PAGE_OFF); // Raise off page  
    motor_gui[board][mot+1].uipnt->gb_motor->setEnabled(false); // disabled 
    // Steppers has  'more'
    mtptr->pb_more->setEnabled(true);
    break;

  case MOT_MODE_DCC:
    switch (motor_gui[board][mot].more_index)
    {
    case 0 : 
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_DCC);
      // we might already have data loaded for it
      set_dcc_gui(board,mot,&(dcc_gui[board][mot]));
      mtptr->pb_more->setText("More");
      break;
    case 1 : 
      mtptr->mode_stack->setCurrentIndex(MOT_PAGE_DCCCONF);
      mtptr->pb_more->setText("Back");
      break;
    }
    // May need to enable odd channel 
    if ((mot==0 || mot==2) && !motor_gui[board][mot+1].uipnt->gb_motor->isEnabled())
        motor_gui[board][mot+1].uipnt->gb_motor->setEnabled(true);
    // has more button 
    mtptr->pb_more->setEnabled(true);
    break;
  }

} // motor_group_mode


//
// Take status from motor data structure
// and update gui so it reflects that status 
//                              0 1 2 3 4 5 6 7 8 9 A B C D E F
const int  mode_to_index[32] ={ 0,1,2,0,0,0,0,0,3,4,0,0,0,0,0,0,
                                0,0,0,0,0,0,0,0,5,6,0,0,0,0,0,0};
void gertbot::set_gui_from_motor(int b,int m) 
{ motor_gui_type *mgp; 
  Ui::motor       *mtpntr;
  QString str;
  int  mode;
  float f_freq;
  mgp = &(motor_gui[b][m]);
  mtpntr = mgp->uipnt;
  // a bit of self protection: 
  if (!board[b].present)
    return;

  // We only want to update the GUI
  // Not have the GUI then send the data to the boards again
  suppress_updates = true;
  mode = mode_to_index[gb_motor[b][m].mode];
  if ((mode & MOT_MODE_STEP_MASK) && (gb_motor[b][m].flags & MOT_STEP_OFF_FLG))
    mode += 2;
  mtpntr->pd_mode->setCurrentIndex(mode);
  mtpntr->pd_ramp_up->setCurrentIndex(gb_motor[b][m].ramp_up);
  mtpntr->pd_ramp_down->setCurrentIndex(gb_motor[b][m].ramp_down);
  mtpntr->pd_ramp_halt->setCurrentIndex(gb_motor[b][m].ramp_halt);
  // Set GUI according to this mode
  set_motormode_gui(b,m);
  end_stop_gui(b,m);
  if (gb_motor[b][m].mode == MOT_MODE_BRUSH)
  { str.sprintf("%d",gb_motor[b][m].frequency);
    mtpntr->le_brushfreq->setText(str);
    // Buttons enable if correct
    if (gb_motor[b][m].frequency>=10 && gb_motor[b][m].frequency<=10000)
      motor_buttons_on(b,m,true);
    else
      motor_buttons_on(b,m,false);
  }
  else
  if (gb_motor[b][m].mode & MOT_MODE_STEP_MASK)
  { f_freq = gb_motor[b][m].frequency/256.0;
    if (f_freq<1000)
      str.sprintf("%4.2f",f_freq);
    else
      str.sprintf("%4.1f",f_freq);
    mtpntr->le_stepfreq->setText(str);
    // Buttons enable if correct
    if (f_freq>=0.0625 && f_freq<=5000)
      motor_buttons_on(b,m,true);
    else
      motor_buttons_on(b,m,false);
  }
  mtpntr->sb_dutycycle->setValue((double)(gb_motor[b][m].duty_cycle/10.00));

  // Set buttons state
  if (gb_motor[b][m].mode == MOT_MODE_BRUSH)
  { 
    if (gb_motor[b][m].move == MOT_MOVE_OFF)
      mtpntr->pb_brushS->setChecked(true);
    else
    { if (gb_motor[b][m].move == MOT_MOVE_A)
        mtpntr->pb_brushA->setChecked(true);
      else
        mtpntr->pb_brushB->setChecked(true);
    }
  } // button DC/brushed 
  else
  { // All buttons up 
    // Todo : color
     mtpntr->pb_stepS->setChecked(false);
     mtpntr->pb_stepA->setChecked(false);
     mtpntr->pb_stepB->setChecked(false);
     if (gb_motor[m][b].steps>0)
       mtpntr->pb_stepA->setStyleSheet("QPushButton { color : green; }");
     if (gb_motor[m][b].steps<0)
       mtpntr->pb_stepB->setStyleSheet("QPushButton { color : green; }");
  } 

  mtpntr->pd_short->setCurrentIndex(gb_motor[b][m].short_hot_stop);

  // enable normal operation again 
  suppress_updates = false;

} // set_gui_from_motor

//
// Given a motor (endmode value) set GUI from it
//
void gertbot::end_stop_gui(int b,int m)
{ 
  Ui::motor       *mtpntr;
  mtpntr = motor_gui[b][m].uipnt;
  if (gb_motor[b][m].endmode & 0x01)
  { mtpntr->cb_endstop_A->setChecked(true);
    j3_gui[b].pd[m*2]->setCurrentIndex( GUI_PD_FUNCT);
  }
  else
    mtpntr->cb_endstop_A->setChecked(false);

  if (gb_motor[b][m].endmode & 0x02)
  { mtpntr->cb_endstop_B->setChecked(true);
    j3_gui[b].pd[m*2+1]->setCurrentIndex( GUI_PD_FUNCT);
  }
  else
    mtpntr->cb_endstop_B->setChecked(false);

  if (gb_motor[b][m].endmode & 0x04)
  { mtpntr->cb_high_A->setChecked(true);
    j3_gui[b].cb[m*2]->setText("High");
    j3_gui[b].cb[m*2]->setChecked(true);
  }
  else
  { mtpntr->cb_high_A->setChecked(false);
    j3_gui[b].cb[m*2]->setText("Low");
    j3_gui[b].cb[m*2]->setChecked(false);
  }

  if (gb_motor[b][m].endmode & 0x08)
  { mtpntr->cb_high_B->setChecked(true);
    j3_gui[b].cb[m*2+1]->setText("High");
    j3_gui[b].cb[m*2+1]->setChecked(true);
  }
  else
  { mtpntr->cb_high_B->setChecked(false);
    j3_gui[b].cb[m*2+1]->setText("Low");
    j3_gui[b].cb[m*2+1]->setChecked(false);
  }
  // set J3 controls/status
  j3pd_endpoint(b,m*2,true); 
  j3pd_endpoint(b,m*2+1,true); 
} // end_stop_gui



//
// Take board data and make GUI accordingly
//
static int pin2adc[4] = { 13,12,15,14}; // pin to ADC controls
void gertbot::set_gui_from_board(int b) 
{ gb_board_type *bpnt;
  int bit;
  uint32_t mask;
  bpnt = &(board[b]); // board+b;
  if (!bpnt->present)
     return; // should never happen 
  suppress_updates = true;

  boardui[b].cb_syncmode->setChecked(bpnt->sync_mode);
  boardui[b].cb_opendrain0->setChecked(bpnt->od & 0x01 ? true : false);
  boardui[b].cb_opendrain1->setChecked(bpnt->od & 0x02 ? true : false);

  // Attention
  boardui[b].pd_attention->setCurrentIndex(bpnt->attentionmode);

  // Need to pop-up/down associated widgets!
  // EXT pd's are set by the motor endstop
  // which status is NOT in the board data structure.
  // Work around: if NOT input and NOT output assume endstop
  for (bit=0; bit<8; bit++)
  { mask = 1<<bit;
    if (bpnt->input_ports & mask)
      j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_INPUT);
    else
    { if (bpnt->output_ports & mask)
        j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_OUTPUT);
     else
       j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_FUNCT);
    }
    j3pd_endpoint(b,bit,false);
  }

  // spares
  for (bit=8; bit<10; bit++)
  { mask = 1<<bit;
    if (bpnt->input_ports & mask)
      j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_INPUT);
    else
       j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_OUTPUT);
    j3pd_spare(b,bit);
  }

  // ADC 0,1,2,3 
  if (bpnt->adc_enable & 0x01)
     j3_gui[b].pd[PD_ADC0]->setCurrentIndex(GUI_PD_FUNCT);
  else
     if (bpnt->input_ports & (1<<10))
       j3_gui[b].pd[PD_ADC0]->setCurrentIndex(GUI_PD_INPUT);
     else
       j3_gui[b].pd[PD_ADC0]->setCurrentIndex(GUI_PD_OUTPUT);

  if (bpnt->adc_enable & 0x02)
     j3_gui[b].pd[PD_ADC1]->setCurrentIndex(GUI_PD_FUNCT);
  else
     if (bpnt->input_ports & (1<<11))
       j3_gui[b].pd[PD_ADC1]->setCurrentIndex(GUI_PD_INPUT);
     else
       j3_gui[b].pd[PD_ADC1]->setCurrentIndex(GUI_PD_OUTPUT);

  if (bpnt->adc_enable & 0x04)
     j3_gui[b].pd[PD_ADC2]->setCurrentIndex(GUI_PD_FUNCT);
  else
     if (bpnt->input_ports & (1<<12))
       j3_gui[b].pd[PD_ADC2]->setCurrentIndex(GUI_PD_INPUT);
     else
       j3_gui[b].pd[PD_ADC2]->setCurrentIndex(GUI_PD_OUTPUT);

  if (bpnt->adc_enable & 0x08)
     j3_gui[b].pd[PD_ADC3]->setCurrentIndex(GUI_PD_FUNCT);
  else
     if (bpnt->input_ports & (1<<13))
       j3_gui[b].pd[PD_ADC3]->setCurrentIndex(GUI_PD_INPUT);
     else
       j3_gui[b].pd[PD_ADC3]->setCurrentIndex(GUI_PD_OUTPUT);
  j3pd_adc(b,PD_ADC0);
  j3pd_adc(b,PD_ADC1);
  j3pd_adc(b,PD_ADC2);
  j3pd_adc(b,PD_ADC3);

  // DAC 0,1  on PD 19,17 
  for (bit=14; bit<16; bit++)
  { mask = 1<<bit;
    if (bpnt->input_ports & mask)
      j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_INPUT);
    else
    { if (bpnt->output_ports & mask)
        j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_OUTPUT);
     else
       j3_gui[b].pd[bit]->setCurrentIndex(GUI_PD_FUNCT);
    }
    j3pd_dac(b,bit);
  }

  suppress_updates = false;
} // set_gui_from_board


//
// Switch GUI between motor and board controls
//
void gertbot::board_mot_slot()
{ QObject *qto = (QObject *)sender();
  int b,m;
  QString str;

  b = qto->objectName()[0].digitValue();

  if (gui_disp_motor[b])
  { // Show board controls
    for (m=0; m<4; m++)
      motor_gui[b][m].motor_frame->hide(); // ????
    board_gui[b].j3_frame->show();
    board_gui[b].misc_group->show();
    board_gui[b].pb_board_to_motor->show();
    board_gui[b].pb_motor_to_board->hide();
    gui_disp_motor[b] = false;
  }
  else
  { // show motor controls
    board_gui[b].j3_frame->hide();
    board_gui[b].misc_group->hide();
    board_gui[b].pb_board_to_motor->hide();
    board_gui[b].pb_motor_to_board->show();
    for (m=0; m<4; m++)
      motor_gui[b][m].motor_frame->show();    
    gui_disp_motor[b] = true;
  }
} // board_mot_slot

//
// J3 pin mode change
// (Pull down activated) 
//
void gertbot::j3pd_slot()
{ QObject *qto = (QObject *)sender();
  int b,p;
  QPalette pal;
  b = qto->objectName()[0].digitValue();
  p = qto->objectName()[1].toAscii()-'0';

  // reset checkbox colour to default: white 
  pal = j3_gui[b].cb[p]->palette();
  pal.setColor( QPalette::Active, QPalette::Base, Qt::white );
  j3_gui[b].cb[p]->setPalette(pal);

  if (p<=7)
    j3pd_endpoint(b,p,false);
  else
  { if (p<=9)
      j3pd_spare(b,p);
    else
      if (p<=13)
        j3pd_adc(b,p);
      else
        j3pd_dac(b,p);
  }
} // j3pd_slot

//
// J3 ext/endpoint mode activated 
// Also called from motor end point control! 
//
void gertbot::j3pd_endpoint(int b,int p,bool from_mot)
{ int mode,high,m;
//  bool endmode_change;
  mode = j3_gui[b].pd[p]->currentIndex();

  switch (mode)
  {
  case GUI_PD_FUNCT : // Endpoint
    j3_gui[b].cb[p]->setCheckable(true);
    // Copy status from motor control
    m = p/2;
    if (p & 1)
    { // switch end mode B on
      if (!from_mot)  motor_gui[b][m].uipnt->cb_endstop_B->setChecked(true);
      high = motor_gui[b][m].uipnt->cb_high_B->isChecked();
    }
    else
    { // switch end mode A on
      if (!from_mot)  motor_gui[b][m].uipnt->cb_endstop_A->setChecked(true);
      high = motor_gui[b][m].uipnt->cb_high_A->isChecked();
    }
    j3_gui[b].cb[p]->setText(high ? "High":"Low");
    j3_gui[b].cb[p]->setToolTip("Checked: endpoint active high, otherwise active low");
    j3_gui[b].cb[p]->setChecked(high ? true : false);

    // Send command to board
    send_endmode_short(b,m); 
    // Micro-controller will set I/O mode when going to end mode 
    break;
  case GUI_PD_INPUT : // input
    m = p/2;
    if (!from_mot)
    {
      if (p & 1)
        motor_gui[b][m].uipnt->cb_endstop_B->setChecked(false);
      else
        motor_gui[b][m].uipnt->cb_endstop_A->setChecked(false);
    }
    send_endmode_short(b,m); 
    j3_set_input(b,p);
    break;
  case GUI_PD_OUTPUT : // output
    m = p/2;
    if (!from_mot)
    {
      if (p & 1)
        motor_gui[b][m].uipnt->cb_endstop_B->setChecked(false);
      else
        motor_gui[b][m].uipnt->cb_endstop_A->setChecked(false);
    }
    send_endmode_short(b,m); 
    j3_set_output(b,p);
    break;
  }
} // j3pd_endpoint

//
// Adapt GUI if spare pin mode changes
//
void gertbot::j3pd_spare(int b,int p)
{ int mode;
  mode = j3_gui[b].pd[p]->currentIndex();
  switch (mode)
  {
  case GUI_PD_INPUT : // input
    j3_set_input(b,p);
    break;
  case GUI_PD_OUTPUT : // output
    j3_set_output(b,p);
    break;
  }
} // j3pd_spare

//
// Adapt GUI if ADC pin mode changes
//
void gertbot::j3pd_adc(int b,int p)
{ int mode,adc;
  mode = j3_gui[b].pd[p]->currentIndex();
  adc = p-10;
  switch (mode)
  {
  case GUI_PD_FUNCT : // ADC
    j3_gui[b].cb[p]->hide();
    j3_gui[b].lbl_adc[adc]->show();
    adc_dac_enable(b);
    break;
  case GUI_PD_INPUT : // input
    j3_gui[b].cb[p]->show();
    j3_gui[b].lbl_adc[adc]->hide();
    adc_dac_enable(b);
    j3_set_input(b,p);
    break;
  case GUI_PD_OUTPUT : // output
    j3_gui[b].cb[p]->show();
    j3_gui[b].lbl_adc[adc]->hide();
    adc_dac_enable(b);
    j3_set_output(b,p);
    break;
  }
} // j3pd_adc

//
// Adapt GUI if DAC pin mode changes
//
void gertbot::j3pd_dac(int b,int p)
{ int mode,dac;
  mode = j3_gui[b].pd[p]->currentIndex();
  dac  = p==14 ? 1 : 0;  
  switch (mode)
  {
  case GUI_PD_FUNCT : // DAC
    j3_gui[b].cb[p]->hide();
    j3_gui[b].sb_dac[dac]->show();
    adc_dac_enable(b);
    break;
  case GUI_PD_INPUT : // input
    j3_gui[b].cb[p]->show();
    j3_gui[b].sb_dac[dac]->hide();
    adc_dac_enable(b);
    j3_set_input(b,p);
    break;
  case GUI_PD_OUTPUT : // output
    j3_gui[b].cb[p]->show();
    j3_gui[b].sb_dac[dac]->hide();
    adc_dac_enable(b);
    j3_set_output(b,p);
    break;
  }
} // j3pd_dac

void gertbot::j3_set_input_cb(int b, int p, int high)
{
  QPalette pal;
  pal = j3_gui[b].cb[p]->palette();
  if (high)
  {
    pal.setColor( QPalette::Active, QPalette::Base, Qt::cyan );
    j3_gui[b].cb[p]->setText("H");
  }
  else
  {
    j3_gui[b].cb[p]->setText("L");
    pal.setColor( QPalette::Active, QPalette::Base, Qt::yellow );
  }
  j3_gui[b].cb[p]->setPalette(pal);
}

//
// Set gui J3 pin to input mode 
//
void gertbot::j3_set_input(int b, int p)
{
  j3_gui[b].cb[p]->setCheckable(false);
  j3_gui[b].cb[p]->setToolTip("1:Input is high, 0: Input is low");
  j3_set_input_cb(b,p,board[b].inputs_status & (1<<p));
  set_io(b);
} // j3_set_input

//
// Set gui J3 pin to output mode 
//
void gertbot::j3_set_output(int b, int p)
{
  j3_gui[b].cb[p]->setCheckable(true);
  if (board[b].outputs_status & (1<<p))
  {
    j3_gui[b].cb[p]->setChecked(true);
    j3_gui[b].cb[p]->setText("1");
  }
  else
  {
    j3_gui[b].cb[p]->setChecked(false);
    j3_gui[b].cb[p]->setText("0");
  }
  j3_gui[b].cb[p]->setToolTip("Checked: Set output high, otherwise set output low");
  set_io(b);
} // j3_set_output

//
// Display various labels for the pins 
//
void gertbot::board_pin_slot()
{ QObject *qto = (QObject *)sender();
  int b;
  b = qto->objectName()[0].digitValue();

  text_display[b] = (text_display[b]+1)%3;

  switch (text_display[b])
  { 
  case 0 : // show pin number names
    boardui[b].lbl_p00 ->setText("2");
    boardui[b].lbl_p01 ->setText("1");
    boardui[b].lbl_p02 ->setText("4");
    boardui[b].lbl_p03 ->setText("3");
    boardui[b].lbl_p04 ->setText("6");
    boardui[b].lbl_p05 ->setText("5");
    boardui[b].lbl_p06 ->setText("8");
    boardui[b].lbl_p07 ->setText("7");
    boardui[b].lbl_p08 ->setText("10");
    boardui[b].lbl_p09 ->setText("9");
    boardui[b].lbl_p010->setText("12");
    boardui[b].lbl_p011->setText("11");
    boardui[b].lbl_p012->setText("14");
    boardui[b].lbl_p013->setText("13");
    boardui[b].lbl_p014->setText("16");
    boardui[b].lbl_p015->setText("15");
    boardui[b].lbl_p016->setText("18");
    boardui[b].lbl_p017->setText("17");
    boardui[b].lbl_p018->setText("20");
    boardui[b].lbl_p019->setText("19");
    break;
  case 1 : // show functional names
    boardui[b].lbl_p00 ->setText("Ext.0");
    boardui[b].lbl_p01 ->setText("Ext.1");
    boardui[b].lbl_p02 ->setText("Ext.2");
    boardui[b].lbl_p03 ->setText("Ext.3");
    boardui[b].lbl_p04 ->setText("Ext.4");
    boardui[b].lbl_p05 ->setText("Ext.5");
    boardui[b].lbl_p06 ->setText("Ext.6");
    boardui[b].lbl_p07 ->setText("Ext.7");
    boardui[b].lbl_p08 ->setText("Spare0");
    boardui[b].lbl_p09 ->setText("Spare1");
    boardui[b].lbl_p010->setText(" - ");
    boardui[b].lbl_p011->setText(" - ");
    boardui[b].lbl_p012->setText("ADC0");
    boardui[b].lbl_p013->setText("ADC1");
    boardui[b].lbl_p014->setText("ADC2");
    boardui[b].lbl_p015->setText("ADC3");
    boardui[b].lbl_p016->setText("DAC1");
    boardui[b].lbl_p017->setText(" - ");
    boardui[b].lbl_p018->setText("DAC0");
    boardui[b].lbl_p019->setText(" - ");
    break;
  case 2 : // endstops
    boardui[b].lbl_p00 ->setText("End0.A");
    boardui[b].lbl_p01 ->setText("End0.B");
    boardui[b].lbl_p02 ->setText("End1.A");
    boardui[b].lbl_p03 ->setText("End1.B");
    boardui[b].lbl_p04 ->setText("End2.A");
    boardui[b].lbl_p05 ->setText("End2.B");
    boardui[b].lbl_p06 ->setText("End3.A");
    boardui[b].lbl_p07 ->setText("End3.B");
    boardui[b].lbl_p08 ->setText("Spare0");
    boardui[b].lbl_p09 ->setText("Spare1");
    boardui[b].lbl_p010->setText(" - ");
    boardui[b].lbl_p011->setText(" - ");
    boardui[b].lbl_p012->setText("ADC0");
    boardui[b].lbl_p013->setText("ADC1");
    boardui[b].lbl_p014->setText("ADC2");
    boardui[b].lbl_p015->setText("ADC3");
    boardui[b].lbl_p016->setText("DAC1");
    boardui[b].lbl_p017->setText(" - ");
    boardui[b].lbl_p018->setText("DAC0");
    boardui[b].lbl_p019->setText(" - ");
    break;

  } // swich 
} // board_pin_slot

//
// Movement buttons on/off 
//
void gertbot::motor_buttons_on(int board,int mot,bool on)
{ Ui::motor *mtpnt;
  mtpnt = motor_gui[board][mot].uipnt;
  if (on)
  {
    mtpnt->pb_brushA->setEnabled(true); 
    mtpnt->pb_brushS->setEnabled(true);
    mtpnt->pb_brushB->setEnabled(true);
    mtpnt->pb_stepA->setEnabled(true); 
    mtpnt->pb_stepS->setEnabled(true);
    mtpnt->pb_stepB->setEnabled(true);
    mtpnt->pb_stepI->setEnabled(true);
  }
  else
  {
    mtpnt->pb_brushA->setEnabled(false); 
    mtpnt->pb_brushS->setEnabled(false);
    mtpnt->pb_brushB->setEnabled(false);
    mtpnt->pb_stepA->setEnabled(false); 
    mtpnt->pb_stepS->setEnabled(false);
    mtpnt->pb_stepB->setEnabled(false);
    mtpnt->pb_stepI->setEnabled(false);
  }
} // motor_buttons_on

//
// Raise related widget
//
void gertbot::more_slot()
{ QObject *qto = (QObject *)sender();
  Ui::motor *mtpnt;
  int b,m;
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  mtpnt =  motor_gui[b][m].uipnt;
  switch (mtpnt->pd_mode->currentIndex())
  {
  case 0 : // off
    return;
  case 1 : // brushed
    motor_gui[b][m].more_index++;
    if (motor_gui[b][m].more_index>1)
      motor_gui[b][m].more_index = 0;
    switch (motor_gui[b][m].more_index)
    {
    case 0 : 
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_BRUSH);
      mtpnt->pb_more->setText("More");
      break;
    case 1 : 
      mtpnt->group_ramp->setEnabled(true);
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_SET);
      mtpnt->pb_more->setText("Back");
      break;
    }
    break; // brushed modes 
  case 2 : // DCC
    motor_gui[b][m].more_index++;
    if (motor_gui[b][m].more_index>1)
      motor_gui[b][m].more_index = 0;
    switch (motor_gui[b][m].more_index)
    {
    case 0 : 
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_DCC);
      mtpnt->pb_more->setText("More");
      break;
    case 1 : 
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_DCCCONF);
      mtpnt->pb_more->setText("Back");
      break;
    }
    return;
  case 3 : // Stepper 
  case 4 : // Stepper 
  case 5 : // Stepper 
  case 6 : // Stepper 
    motor_gui[b][m].more_index++;
    if (motor_gui[b][m].more_index>1)
      motor_gui[b][m].more_index = 0;
    switch (motor_gui[b][m].more_index)
    {
    case 0 : 
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_STEP);
      mtpnt->pb_more->setText("More");
      break;
    case 1 : 
      mtpnt->mode_stack->setCurrentIndex(MOT_PAGE_SET);
      mtpnt->group_ramp->setEnabled(false);
      mtpnt->pb_more->setText("Back");
      break;
    }
    break; // stepper modes 

  }
} // more_slot


