//
// Gertbot GUI
//
// Motors control 
//


#include <stdio.h>

#include "gertbot.h"

extern bool uart_opened;

//
// Endstop mode changed 
//
void gertbot::endmode_slot()
{ QObject *qto = (QObject *)sender();
  int b,m,c,p;
  bool checked;
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
//  send_endmode_short(b,m);
  //
  // Update J3 gui
  // This is a bit of a mess and really needs cleaning up 
  // Controlling I/O from two places (motors and J3) and keeping
  // them consistent needs better solution
  // 
  // Which one changed?
  c = qto->objectName()[2].digitValue();
  p = (m*2);
  // When end mode is switched off currently always go to input mode 
  switch (c)
  {
  case 0 :  // A changed
      checked = motor_gui[b][m].uipnt->cb_endstop_A->isChecked();
      // set board control to this mode
      j3_gui[b].pd[p]->setCurrentIndex( checked ? GUI_PD_FUNCT : GUI_PD_INPUT);
      j3pd_endpoint(b,p,true); // Let board code handle it 
      break;
  case 1 :  // B changed 
     checked = motor_gui[b][m].uipnt->cb_endstop_B->isChecked();
      // set board control to this mode
      j3_gui[b].pd[p+1]->setCurrentIndex( checked ? GUI_PD_FUNCT : GUI_PD_INPUT);
      j3pd_endpoint(b,p+1,true); // Let board code handle it 
      break;
  }

  switch (c)
  {
  case 2 :  checked = motor_gui[b][m].uipnt->cb_endstop_A->isChecked();
            if (checked)
            {
              checked = motor_gui[b][m].uipnt->cb_high_A->isChecked();
              j3_gui[b].cb[p]->setText(checked ? "High":"Low");
              suppress_updates = true;
              j3_gui[b].cb[p]->setChecked(checked);
              suppress_updates = false;
              send_endmode_short(b,m); 
            }
            break;
  case 3 :  checked = motor_gui[b][m].uipnt->cb_endstop_B->isChecked();
            if (checked)
            { checked = motor_gui[b][m].uipnt->cb_high_B->isChecked();
              j3_gui[b].cb[p+1]->setText(checked ? "High":"Low");
              suppress_updates = true;
              j3_gui[b].cb[p+1]->setChecked(checked);
              suppress_updates = false;
              send_endmode_short(b,m); 
            }
            break;
  }

} // endmode_slot

//
// Send end mode command
// command is derived from the motor control settings 
//
void gertbot::send_endmode_short(int b,int m)
{ Ui::motor *mtpntr;
  byte id,mode;
  mtpntr = motor_gui[b][m].uipnt;

  mode  = mtpntr->cb_endstop_A->isChecked() ? 0x01 : 0x00;
  mode |= mtpntr->cb_endstop_B->isChecked() ? 0x02 : 0x00;
  mode |= mtpntr->cb_high_A->isChecked() ? 0x04 : 0x00;
  mode |= mtpntr->cb_high_B->isChecked() ? 0x08 : 0x00;
  gb_motor[b][m].endmode = mode;
  id = (b<<2) | m;

  mode |= (mtpntr->pd_short->currentIndex()<<4);
  if (suppress_updates)
    return;
  
  // Todo: supress if neither endmode nor short mode has changed
  uart_tx((byte)CMD_STOPSHORT,id,mode);
} // endmode_slot

//
// Frequency changed 
//
void gertbot::freq_slot()
{ int b,m;
  QObject *qto = (QObject *)sender();
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  send_freq(b,m);
}


void gertbot::send_freq(int b,int m)
{ int i_freq;
  float f_freq;
  bool  ok;
  byte id,freq[3];
  Ui::motor *mtrpnt; // 
  mtrpnt = motor_gui[b][m].uipnt;
  id = (b<<2) | m;
  if (mtrpnt->pd_mode->currentIndex()==1)
  { // linear frequency
    i_freq = mtrpnt->le_brushfreq->text().toInt(&ok);
    if (!ok || i_freq<10 || i_freq>30000)
    { // Illegal value
      mtrpnt->le_brushfreq->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
      return;
    }
    mtrpnt->le_brushfreq->setStyleSheet("QLineEdit { background-color : white; color : black; }");
    freq[0] = (byte)((i_freq>>8)& 0xFF);
    freq[1] = (byte)( i_freq    & 0xFF);
    uart_tx((byte)CMD_LINFREQ,id,freq[0],freq[1]);
    motor_buttons_on(b,m,true);
  }
  else
  { // must be step frequency
    f_freq = mtrpnt->le_stepfreq->text().toFloat(&ok);
    if (!ok || f_freq<0.0625 || f_freq>5000)
    { // Illegal value
      mtrpnt->le_stepfreq->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
      return;
    }
    mtrpnt->le_stepfreq->setStyleSheet("QLineEdit { background-color : white; color : black; }");
    i_freq = (int) (f_freq * 256);
    freq[0] = (byte)((i_freq>>16)& 0xFF);
    freq[1] = (byte)((i_freq>>8)& 0xFF);
    freq[2] = (byte)( i_freq    & 0xFF);
    uart_tx((byte)CMD_STEPFREQ,id,freq[0],freq[1],freq[2]);
    motor_buttons_on(b,m,true);
  }
} // freq_slot

//
// Duty cycle changed 
//
void gertbot::dutycycle_slot()
{ int b,m;
  QObject *qto = (QObject *)sender();
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  send_dutycycle(b,m);
}

//
// Send duty cycle command to board/motor
//
void gertbot::send_dutycycle(int b,int m)
{ int i_dc;
  double d_dc;
  byte id,dc[2];
  Ui::motor *mtptr; // 
  id = (b<<2) | m;
  mtptr = motor_gui[b][m].uipnt;
  d_dc = mtptr->sb_dutycycle->value();
  i_dc = (int)(d_dc*10);
  if (i_dc<0 || i_dc>1000)
  { // Illegal value
    mtptr->sb_dutycycle->setStyleSheet("QDoubleSpinBox { background-color : yellow; color : red; }");
    return;
  }
  mtptr->sb_dutycycle->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  dc[0] = (byte)((i_dc>>8)& 0xFF);
  dc[1] = (byte)( i_dc    & 0xFF);
  uart_tx((byte)CMD_LINDC,id,dc[0],dc[1]);
} // dutycycle_slot


//
// Steps changed 
// Checking value only 
//
void gertbot::steps_slot()
{ int b,m,steps;
  bool  ok;
  QObject *qto = (QObject *)sender();
  Ui::motor *mtptr; // 
  if (suppress_updates)
    return; 
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  mtptr = motor_gui[b][m].uipnt;
  steps = mtptr->le_steps->text().toInt(&ok);
  if (!ok || steps<0 || steps>0x7FFFFF)
  { // Illegal value
    mtptr->le_steps->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
    return;
  }
  mtptr->le_steps->setStyleSheet("QLineEdit { background-color : white; color : black; }");
} // steps_slot

//
// Run push buttons
// 
void gertbot::runA_slot() { run_slot(1); }
void gertbot::runS_slot() { run_slot(2); }
void gertbot::runB_slot() { run_slot(3); }
void gertbot::runI_slot() { run_slot(4); }

void gertbot::run_slot(int but)
{ QObject *qto = (QObject *)sender();
  int b,m;
  QString str;
  gb_motor_type mot_status;
  if (suppress_updates)
    return; 

  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  send_run(b,m,but);
  if (gb_motor[b][m].mode & MOT_MODE_STEP_MASK)
  { // stepper mode 
    // get motor status to display how many steps where left 
    if (get_motor_missed(b,m,&mot_status))
    { str.sprintf("Missed: %d",mot_status.missed);
      motor_gui[b][m].uipnt->lbl_step_status->setText(str);
    }
  }
} // run_slot

//
// Read GUI status and use it to send a 
// motor run command 
//
void gertbot::send_run(int b,int m,int but)
{ byte id,mode,st[3];
  int steps;
  bool ok;
  motor_gui_type *mgp; // 
  Ui::motor *mtptr;
  mgp = &(motor_gui[b][m]);
  mtptr = mgp->uipnt;
  id = (b<<2) | m;
  if (mtptr->pd_mode->currentIndex()==1)
  { // Brushed mode
    switch (but)
    {
    case 1 : mode = MOT_MOVE_A; break;
    case 2 : mode = MOT_PWR_OFF; break;
    case 3 : mode = MOT_MOVE_B; break;
    }
    uart_tx((byte)CMD_LINON,id,mode);
  }
  else
  { // stepper mode 
    // cheating: Setting all to black then ONE to green
    mtptr->pb_stepA->setIcon(icon_left);
    mtptr->pb_stepS->setIcon(icon_pause);
    mtptr->pb_stepB->setIcon(icon_right);
    mtptr->pb_stepI->setIcon(icon_stop);
    steps = 0x00; // prevent 'used but not set' warning
    switch (but)
    {
    case 1 : // positive 
      steps = mtptr->le_steps->text().toInt(&ok);
      if (!ok || steps<0 || steps>0x7FFFFF)
        return;
      // Indicate A was last pressed 
      mtptr->pb_stepA->setIcon(icon_left_grn);
      break;
    case 2 : // Stop 
       // Indicate S was last pressed 
      mtptr->pb_stepS->setIcon(icon_pause_grn);
      break;
    case 3 : // negative 
      steps = mtptr->le_steps->text().toInt(&ok);
      if (!ok || steps<0 || steps>0x7FFFFF)
        return;
      steps = -steps;
      // Indicate B was last pressed 
      mtptr->pb_stepB->setIcon(icon_right_grn);
      break;
    case 4 : // Idle 
      // Indicate I was last pressed 
      mtptr->pb_stepI->setIcon(icon_stop_grn);
      break;
    }
    if (but==4)
    { // send brushed motor OFF command 
      uart_tx((byte)CMD_LINON,id,MOT_PWR_OFF);
    }
    else
    { // send steps command 
      st[0] = (byte)((steps>>16) & 0xFF);
      st[1] = (byte)((steps>> 8) & 0xFF);
      st[2] = (byte)( steps      & 0xFF);
      uart_tx((byte)CMD_STEP,id,st[0],st[1],st[2]);
    }
  }
} // run_slot


// Send stop all command 
void gertbot::stop_all_slot()
{
  uart_tx((byte)CMD_STOPALL,(byte)CMD_STOP2ND);
  set_stop_buttons(0);
}

// Send halt command 
void gertbot::halt_slot()
{
  uart_tx((byte)CMD_PWR_OFF,(byte)CMD_STOP2ND);
  set_stop_buttons(1);
}

void gertbot::do_sync_slot()
{
  if (uart_opened)
   uart_tx((byte)CMD_SYNC,(byte)0x18);
} // do_sync_slot

//
// Set all motor movement buttons to stop
// halt=1 means no power
// 
void gertbot::set_stop_buttons(int halt)
{ int b,m;
  motor_gui_type *mgp; // 
  Ui::motor *mtptr;
  suppress_updates = true;
  for (b=0; b<4; b++)
  { if (board[b].present)
    { for (m=0; m<4; m++)
      { 
        mgp = &(motor_gui[b][m]);
        mtptr = mgp->uipnt;
        switch (mtptr->pd_mode->currentIndex())
        {
        case 1 : // MOT_MODE_BRUSH
           mtptr->pb_brushS->setChecked(true);
           mtptr->pb_brushA->setChecked(false);
           mtptr->pb_brushB->setChecked(false);
           break;
        case 2: // MOT_MODE_STEP_GRAY
        case 3: // MOT_MODE_STEP_PULSE
           mtptr->pb_stepA->setIcon(icon_left);
           mtptr->pb_stepB->setIcon(icon_right);
           mtptr->pb_stepS->setIcon(halt ? icon_pause : icon_pause_grn);
           mtptr->pb_stepI->setIcon(halt ? icon_stop_grn : icon_stop);
           break;
        } // motor mode 
      } // for each motor
    } // if present
  } // for each board
  suppress_updates = false;
} // set_stop_buttons

//
// Get motor configuration from board
//
// returns 13 bytes:
// ID, CMD, mode, endstop+short, 
// freq:MS, freq:MM, freq:LS, 
// duty-cycle:MS, duty-cycle:LS,
// Ramp-up+Ramp-down,  Ramp-halt,
// 0, 0 
bool gertbot::get_motor_config(int brd,int mot,gb_motor_type *status)
{ byte id,rec[16],cmd[2];
  QString str;
 
  id = (brd<<2) | mot;
  cmd[0] = CMD_MOT_CONFIG;
  cmd[1] = id;
  uart_tx(cmd,2,16);
  // Get reply...
  if (!get_reply(rec,13,id,(byte)CMD_MOT_CONFIG))
  { str.sprintf("%d:No reply on read Motor %d",brd,mot);
    add_log(str);
    return false;
  }
  status->mode            =rec[2] & 0x0F;
  if (rec[2] & 0x10)
    status->flags |=  MOT_STEP_OFF_FLG;
  else
    status->flags &= ~MOT_STEP_OFF_FLG;

  status->endmode         =rec[3] & 0xF;
  status->short_hot_stop  =rec[3]>>4;

  status->frequency       =(rec[4]<<16) + (rec[5]<<8)+rec[6];
  status->duty_cycle      =(rec[7]<<8)+rec[8];
  status->ramp_up         =(rec[9] & 0x0F);
  status->ramp_down       =(rec[9] >>4);
  status->ramp_halt       =(rec[10] & 0x0F);
  return true;
} // get_motor_config

//
// Get motor status from board
//
// returns 6 bytes:
// ID, CMD, move, 
// step_count:MS, step_count:MM, step_count:LS
//
bool gertbot::get_motor_status(int brd,int mot,gb_motor_type *status)
{ byte id,rec[9],cmd[2];
  QString str;
 
  id = (brd<<2) | mot;
  cmd[0] = CMD_MOT_STATUS;
  cmd[1] = id;
  uart_tx(cmd,2,16);
  // Get reply...
  if (!get_reply(rec,6,id,(byte)CMD_MOT_STATUS))
  { str.sprintf("%d:No reply on read Motor %d",brd,mot);
    add_log(str);
    return false;
  }
  status->move   =rec[2] & 0x0F;
  status->remain =(rec[3]<<16) + (rec[4]<<8)+rec[5];

  return true;
} // get_motor_status

//
// Get motor missed steps from board
//
// returns 5 bytes:
// ID, CMD, 
// missed:MS, missed:MM, missed:LS
//
bool gertbot::get_motor_missed(int brd,int mot,gb_motor_type *status)
{ byte id,rec[9],cmd[2];
  QString str;
  id = (brd<<2) | mot;
  cmd[0] = CMD_MOT_MISSED;
  cmd[1] = id;
  uart_tx(cmd,2,16);
  // Get reply...
  if (!get_reply(rec,8,id,(byte)CMD_MOT_MISSED))
  { str.sprintf("%d:No reply on read Motor %d",brd,mot);
    add_log(str);
    return false;
  }
  status->missed =(rec[2]<<16) + (rec[3]<<8)+rec[4];

  return true;
} // get_motor_missed

void gertbot::ramp_slot()
{ QObject *qto = (QObject *)sender();
  int b,m;
  Ui::motor *mtptr;
  byte id,b1,b2;
  b = qto->objectName()[0].digitValue();
  m = qto->objectName()[1].digitValue();
  mtptr = motor_gui[b][m].uipnt;
  gb_motor[b][m].ramp_up   = mtptr->pd_ramp_up->currentIndex();
  gb_motor[b][m].ramp_down = mtptr->pd_ramp_down->currentIndex();
  gb_motor[b][m].ramp_halt = mtptr->pd_ramp_halt->currentIndex();
  id = (b<<2) | m;
  b1 = gb_motor[b][m].ramp_up | (gb_motor[b][m].ramp_down<<4);
  b2 = gb_motor[b][m].ramp_halt;
  uart_tx((byte)CMD_SET_RAMP,id,b1,b2);

} // ramp_slot




